"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AssetService = void 0;
const kuzzle_1 = require("kuzzle");
const kuzzle_plugin_commons_1 = require("kuzzle-plugin-commons");
const lodash_1 = __importDefault(require("lodash"));
const plugin_1 = require("../plugin");
const shared_1 = require("../shared");
const AssetSerializer_1 = require("./model/AssetSerializer");
class AssetService extends shared_1.DigitalTwinService {
    constructor(plugin, assetHistoryService) {
        super(plugin, plugin_1.InternalCollection.ASSETS);
        this.assetHistoryService = assetHistoryService;
    }
    registerAskEvents() {
        super.registerAskEvents();
        (0, kuzzle_plugin_commons_1.onAsk)("ask:device-manager:asset:refresh-model", this.refreshModel.bind(this));
    }
    async get(engineId, assetId, request) {
        return this.getDocument(request, assetId, {
            collection: plugin_1.InternalCollection.ASSETS,
            engineId,
        });
    }
    /**
     * Update an asset metadata
     */
    async update(engineId, assetId, metadata, request) {
        return (0, shared_1.lock)(`asset:${engineId}:${assetId}`, async () => {
            const asset = await this.get(engineId, assetId, request);
            const updatedPayload = await this.app.trigger("device-manager:asset:update:before", { asset, metadata });
            const updatedAsset = await this.updateDocument(request, {
                _id: assetId,
                _source: { metadata: updatedPayload.metadata },
            }, {
                collection: plugin_1.InternalCollection.ASSETS,
                engineId,
            }, { source: true });
            await this.assetHistoryService.add(engineId, [
                {
                    asset: updatedAsset._source,
                    event: {
                        metadata: {
                            names: Object.keys((0, shared_1.flattenObject)(updatedPayload.metadata)),
                        },
                        name: "metadata",
                    },
                    id: updatedAsset._id,
                    timestamp: Date.now(),
                },
            ]);
            await this.app.trigger("device-manager:asset:update:after", {
                asset: updatedAsset,
                metadata: updatedPayload.metadata,
            });
            return updatedAsset;
        });
    }
    /**
     * Replace an asset metadata
     */
    async replaceMetadata(engineId, assetId, metadata, request) {
        return (0, shared_1.lock)(`asset:${engineId}:${assetId}`, async () => {
            const asset = await this.get(engineId, assetId, request);
            const unknownMetadata = {};
            for (const key in metadata) {
                if (key in asset._source.metadata) {
                    asset._source.metadata[key] = metadata[key];
                }
                else {
                    unknownMetadata[key] = metadata[key];
                }
            }
            // ? If metadata key is unknown on the asset we check that it exists in the assetModel mappings
            if (Object.keys(unknownMetadata).length > 0) {
                const assetModel = await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:model:asset:get", { engineGroup: engineId.split("-")[1], model: asset._source.model });
                for (const key in unknownMetadata) {
                    if (key in assetModel.asset.metadataMappings) {
                        asset._source.metadata[key] = unknownMetadata[key];
                    }
                }
            }
            const updatedPayload = await this.app.trigger("device-manager:asset:update:before", { asset, metadata });
            const updatedAsset = await this.sdk.document.replace(engineId, plugin_1.InternalCollection.ASSETS, assetId, updatedPayload.asset._source, { triggerEvents: true });
            await this.assetHistoryService.add(engineId, [
                {
                    asset: updatedAsset._source,
                    event: {
                        metadata: {
                            names: Object.keys((0, shared_1.flattenObject)(updatedPayload.metadata)),
                        },
                        name: "metadata",
                    },
                    id: updatedAsset._id,
                    timestamp: Date.now(),
                },
            ]);
            await this.app.trigger("device-manager:asset:update:after", {
                asset: updatedAsset,
                metadata: updatedPayload.metadata,
            });
            return updatedAsset;
        });
    }
    /**
     * Update or Create an asset metadata
     */
    async upsert(engineId, model, reference, metadata, request) {
        const assetId = AssetSerializer_1.AssetSerializer.id(model, reference);
        return (0, shared_1.lock)(`asset:${engineId}:${assetId}`, async () => {
            const asset = await this.get(engineId, assetId, request).catch(() => null);
            if (!asset) {
                return this._create(assetId, engineId, model, reference, metadata, request);
            }
            const updatedPayload = await this.app.trigger("device-manager:asset:update:before", { asset, metadata });
            const updatedAsset = await this.updateDocument(request, {
                _id: assetId,
                _source: { metadata: updatedPayload.metadata },
            }, {
                collection: plugin_1.InternalCollection.ASSETS,
                engineId,
            }, { source: true });
            await this.assetHistoryService.add(engineId, [
                {
                    asset: updatedAsset._source,
                    event: {
                        metadata: {
                            names: Object.keys((0, shared_1.flattenObject)(updatedPayload.metadata)),
                        },
                        name: "metadata",
                    },
                    id: updatedAsset._id,
                    timestamp: Date.now(),
                },
            ]);
            await this.app.trigger("device-manager:asset:update:after", {
                asset: updatedAsset,
                metadata: updatedPayload.metadata,
            });
            return updatedAsset;
        });
    }
    async _create(assetId, engineId, model, reference, metadata, request) {
        const engine = await this.getEngine(engineId);
        const assetModel = await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:model:asset:get", { engineGroup: engine.group, model });
        const assetMetadata = {};
        for (const metadataName of Object.keys(assetModel.asset.metadataMappings)) {
            assetMetadata[metadataName] = null;
        }
        for (const [metadataName, metadataValue] of Object.entries(assetModel.asset.defaultMetadata)) {
            lodash_1.default.set(assetMetadata, metadataName, metadataValue);
        }
        const measures = {};
        for (const { name } of assetModel.asset.measures) {
            measures[name] = null;
        }
        const asset = await this.createDocument(request, {
            _id: assetId,
            _source: {
                groups: [],
                lastMeasuredAt: null,
                linkedDevices: [],
                measures,
                metadata: { ...assetMetadata, ...metadata },
                model,
                modelLocales: assetModel.asset.locales,
                reference,
                softTenant: [],
            },
        }, {
            collection: plugin_1.InternalCollection.ASSETS,
            engineId,
        });
        await this.assetHistoryService.add(engineId, [
            {
                asset: asset._source,
                event: {
                    metadata: {
                        names: Object.keys((0, shared_1.flattenObject)(asset._source.metadata)),
                    },
                    name: "metadata",
                },
                id: asset._id,
                timestamp: Date.now(),
            },
        ]);
        return asset;
    }
    /**
     * Create an asset metadata
     */
    async create(engineId, model, reference, metadata, request) {
        const assetId = AssetSerializer_1.AssetSerializer.id(model, reference);
        return (0, shared_1.lock)(`asset:${engineId}:${assetId}`, async () => this._create(assetId, engineId, model, reference, metadata, request));
    }
    /**
     * Delete an asset metadata
     */
    async delete(engineId, assetId, request) {
        const user = request.getUser();
        const strict = request.getBoolean("strict");
        return (0, shared_1.lock)(`asset:${engineId}:${assetId}`, async () => {
            const asset = await this.get(engineId, assetId, request);
            if (strict && asset._source.linkedDevices.length !== 0) {
                throw new kuzzle_1.BadRequestError(`Asset "${assetId}" is still linked to devices.`);
            }
            for (const { _id: deviceId } of asset._source.linkedDevices) {
                await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:device:unlink-asset", { deviceId, user });
            }
            await this.deleteDocument(request, assetId, {
                collection: plugin_1.InternalCollection.ASSETS,
                engineId,
            });
        });
    }
    async search(engineId, searchParams, request) {
        return this.searchDocument(request, searchParams, {
            collection: plugin_1.InternalCollection.ASSETS,
            engineId,
        });
    }
    async migrateTenant(user, assetsList, engineId, newEngineId) {
        let errors = [];
        let successes = [];
        //Sanity check
        if (assetsList.length === 0) {
            throw new kuzzle_1.BadRequestError("No assets to migrate");
        }
        await (0, shared_1.lock)(`engine:${engineId}:${newEngineId}`, async () => {
            if (!user.profileIds.includes("admin")) {
                throw new kuzzle_1.BadRequestError(`User ${user._id} is not authorized to migrate assets`);
            }
            // check if tenant destination is in the same group
            const engine = await this.getEngine(engineId);
            const newEngine = await this.getEngine(newEngineId);
            if (engine.group !== newEngine.group) {
                throw new kuzzle_1.BadRequestError(`Engine ${newEngineId} is not in the same group as ${engineId}`);
            }
            //First of all, as mCreate seems to be buggy, ensure some assets don't
            //already exists in the destination tenant
            const assetsCheck = await this.sdk.document.mGet(newEngineId, plugin_1.InternalCollection.ASSETS, assetsList);
            const assetsCheckedIdExisting = assetsCheck.successes.map((a) => a._id);
            errors = [...assetsCheckedIdExisting];
            //Get all assets to migrate
            const assetsCheckedList = assetsList.filter((id) => !assetsCheckedIdExisting.includes(id));
            if (assetsCheckedList.length === 0) {
                throw new kuzzle_1.BadRequestError("All assets to migrate already exists in destination tenant.");
            }
            const assets = await this.sdk.document.mGet(engineId, plugin_1.InternalCollection.ASSETS, assetsCheckedList);
            errors = errors.concat(...assets.errors);
            if (assets.successes.length === 0) {
                this.app.log.error("No assets found to migrate");
                return { errors, successes };
            }
            //Get all the asset content, in order to create thm by batch
            const assetsContent = assets.successes.map((asset) => ({
                _id: asset._id,
                body: asset._source,
            }));
            //We want to create the new asset with linked devices and groups empty
            const assetsContentCopy = lodash_1.default.cloneDeep(assetsContent);
            for (const asset of assetsContentCopy) {
                asset.body.linkedDevices = [];
                asset.body.groups = [];
            }
            //Even if they exist in the destination tenant, try creating them all
            //using batch
            const assetsCreated = await this.sdk.document.mCreate(newEngineId, plugin_1.InternalCollection.ASSETS, assetsContentCopy);
            //We consider here we will return as success what we have been able
            //to create, and related errors
            const assetsCreatedId = assetsCreated.successes.map((a) => a._id);
            const assetsNotCreatedId = assetsCreated.errors.map((a) => a.document._id);
            successes = [...assetsCreatedId];
            errors = errors.concat(...assetsNotCreatedId);
            //Iterate over all created asset, and migrate each one
            for (const asset of assetsCreated.successes) {
                //We need to recover the linked devices to this asset,
                //because we reset them when we create the asset
                const assetOriginal = assets.successes.find((a) => a._id === asset._id);
                // get linked devices to this asset, if any
                const linkedDevices = assetOriginal._source.linkedDevices.map((d) => ({
                    _id: d._id,
                    measureNames: d.measureNames,
                }));
                // ... and iterate over this list
                for (const device of linkedDevices) {
                    // detach linked devices from current tenant (it also unkinks asset)
                    await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:device:detach-engine", { deviceId: device._id, user });
                    // ... and attach to new tenant
                    await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:device:attach-engine", { deviceId: device._id, engineId: newEngineId, user });
                    // ... and link this device to the asset in the new tenant
                    await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:device:link-asset", {
                        assetId: asset._id,
                        deviceId: device._id,
                        engineId: newEngineId,
                        measureNames: device.measureNames,
                        user,
                    });
                }
            }
            // Finally here, we can delete the newly create assets in the source engine !
            await this.sdk.document.mDelete(engineId, plugin_1.InternalCollection.ASSETS, assetsCreatedId);
            //Refresh ES indexes and collections
            const collectionsToRefresh = [
                {
                    collection: plugin_1.InternalCollection.ASSETS,
                    index: engineId,
                },
                {
                    collection: plugin_1.InternalCollection.DEVICES,
                    index: engineId,
                },
                {
                    collection: plugin_1.InternalCollection.ASSETS,
                    index: newEngineId,
                },
                {
                    collection: plugin_1.InternalCollection.DEVICES,
                    index: newEngineId,
                },
                {
                    collection: plugin_1.InternalCollection.DEVICES,
                    index: this.config.adminIndex,
                },
            ].map(({ index, collection }) => {
                return this.sdk.collection.refresh(index, collection);
            });
            await Promise.all(collectionsToRefresh);
        });
        return { errors, successes };
    }
    /**
     * Replace an asset metadata
     */
    async mReplaceAndHistorize(engineId, assets, removedMetadata, { refresh }) {
        const replacedAssets = await this.sdk.document.mReplace(engineId, plugin_1.InternalCollection.ASSETS, assets.map((asset) => ({ _id: asset._id, body: asset._source })), { refresh, source: true });
        const histories = replacedAssets.successes.map((asset) => ({
            asset: asset._source,
            event: {
                metadata: {
                    names: Object.keys((0, shared_1.flattenObject)(asset._source.metadata)),
                },
                name: "metadata",
            },
            id: asset._id,
            timestamp: Date.now(),
        }));
        await this.assetHistoryService.add(engineId, histories);
        return replacedAssets;
    }
    /**
     * Retrieve locales with the specified model and search all assets related to the model to update the locales.
     * The operation is historized.
     * @param request
     * @param engineGroup
     * @param model
     */
    async updateModelLocales(request, engineGroup, model) {
        let resultModel;
        try {
            const { result: fetchedResult } = await this.sdk.query({
                action: "getAsset",
                body: {},
                controller: "device-manager/models",
                engineGroup,
                model,
            });
            resultModel = fetchedResult;
        }
        catch (e) {
            throw new kuzzle_1.BadRequestError(`${e} Verify in your arguments that the asset model belongs to the engineGroup "${engineGroup}"`, {}, 400);
        }
        const locales = resultModel._source.asset.locales;
        const engines = await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:engine:list", {
            group: engineGroup,
        });
        const results = [];
        for (const engine of engines) {
            const resultUpdateByQuery = await this.sdk.document.updateByQuery(engine.index, "assets", {
                bool: {
                    must: [
                        {
                            term: {
                                model: model,
                            },
                        },
                    ],
                },
            }, {
                modelLocales: locales,
            });
            results.push({
                engineIndex: engine.index,
                result: {
                    errors: [...resultUpdateByQuery.errors],
                    successes: [...resultUpdateByQuery.successes],
                },
            });
        }
        const errorsFiltered = results.filter((re) => re.result.errors.length > 0);
        if (errorsFiltered.length === results.length) {
            throw new kuzzle_1.BadRequestError("All the assets failed to be updated", {}, 400);
        }
        if (errorsFiltered.length < results.length && errorsFiltered.length !== 0) {
            throw new kuzzle_1.PartialError("Some assets failed to be updated", errorsFiltered, {}, 206);
        }
        return results;
    }
    async getEngine(engineId) {
        const engine = await this.sdk.document.get(this.config.adminIndex, plugin_1.InternalCollection.CONFIG, `engine-device-manager--${engineId}`);
        return engine._source.engine;
    }
    async refreshModel({ assetModel, }) {
        const engines = await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:engine:list", {
            group: assetModel.engineGroup,
        });
        const targets = engines.map((engine) => ({
            collections: [plugin_1.InternalCollection.ASSETS],
            index: engine.index,
        }));
        const assets = await this.sdk.query({
            action: "search",
            body: { query: { equals: { model: assetModel.asset.model } } },
            controller: "document",
            lang: "koncorde",
            targets,
        });
        const modelMetadata = {};
        for (const metadataName of Object.keys(assetModel.asset.metadataMappings)) {
            const defaultMetadata = assetModel.asset.defaultMetadata[metadataName];
            modelMetadata[metadataName] = defaultMetadata ?? null;
        }
        const removedMetadata = [];
        const updatedAssetsPerIndex = assets.result.hits.reduce((acc, asset) => {
            const assetMetadata = { ...asset._source.metadata };
            for (const key of Object.keys(asset._source.metadata)) {
                if (!(key in modelMetadata)) {
                    removedMetadata.push(key);
                    delete assetMetadata[key];
                }
            }
            asset._source.metadata = {
                ...modelMetadata,
                ...assetMetadata,
            };
            acc[asset.index].push(asset);
            return acc;
        }, Object.fromEntries(engines.map((engine) => [
            engine.index,
            [],
        ])));
        await Promise.all(Object.entries(updatedAssetsPerIndex).map(([index, updatedAssets]) => this.mReplaceAndHistorize(index, updatedAssets, removedMetadata, {
            refresh: "wait_for",
        })));
    }
}
exports.AssetService = AssetService;
//# sourceMappingURL=AssetService.js.map