"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AssetsGroupsController = void 0;
const kuzzle_1 = require("kuzzle");
const kuzzle_plugin_commons_1 = require("kuzzle-plugin-commons");
const plugin_1 = require("../plugin");
class AssetsGroupsController {
    constructor(plugin, assetsGroupsService) {
        this.plugin = plugin;
        this.assetsGroupsService = assetsGroupsService;
        /* eslint-disable sort-keys */
        this.definition = {
            actions: {
                create: {
                    handler: this.create.bind(this),
                    http: [
                        {
                            path: "device-manager/:engineId/assetsGroups/:_id",
                            verb: "post",
                        },
                    ],
                },
                get: {
                    handler: this.get.bind(this),
                    http: [
                        { path: "device-manager/:engineId/assetsGroups/:_id", verb: "get" },
                    ],
                },
                update: {
                    handler: this.update.bind(this),
                    http: [
                        { path: "device-manager/:engineId/assetsGroups/:_id", verb: "put" },
                    ],
                },
                delete: {
                    handler: this.delete.bind(this),
                    http: [
                        {
                            path: "device-manager/:engineId/assetsGroups/:_id",
                            verb: "delete",
                        },
                    ],
                },
                search: {
                    handler: this.search.bind(this),
                    http: [
                        {
                            path: "device-manager/:engineId/assetsGroups/_search",
                            verb: "get",
                        },
                        {
                            path: "device-manager/:engineId/assetsGroups/_search",
                            verb: "post",
                        },
                    ],
                },
                addAsset: {
                    handler: this.addAsset.bind(this),
                    http: [
                        {
                            path: "device-manager/:engineId/assetsGroups/:_id/addAsset",
                            verb: "post",
                        },
                    ],
                },
                removeAsset: {
                    handler: this.removeAsset.bind(this),
                    http: [
                        {
                            path: "device-manager/:engineId/assetsGroups/:_id/removeAsset",
                            verb: "post",
                        },
                    ],
                },
            },
        };
        /* eslint-enable sort-keys */
    }
    get sdk() {
        return this.plugin.context.accessors.sdk;
    }
    get as() {
        return (user) => {
            if (user?._id) {
                return this.sdk.as(user, { checkRights: false });
            }
            return this.sdk;
        };
    }
    async checkParent(engineId, parent) {
        if (typeof parent !== "string") {
            return;
        }
        try {
            const assetGroup = await this.sdk.document.get(engineId, plugin_1.InternalCollection.ASSETS_GROUPS, parent);
            if (assetGroup._source.parent !== null) {
                throw new kuzzle_1.BadRequestError(`Can't create asset group with more than one nesting level`);
            }
        }
        catch (error) {
            if (error.status === 404) {
                throw new kuzzle_1.BadRequestError(`The parent group "${parent}" does not exist`);
            }
            throw error;
        }
    }
    async checkChildren(engineId, children) {
        if (!Array.isArray(children)) {
            throw new kuzzle_1.BadRequestError("The Children property should be an array");
        }
        if (children.length === 0) {
            return;
        }
        const { result } = await this.sdk.query({
            action: "mExists",
            body: {
                ids: children,
            },
            collection: plugin_1.InternalCollection.ASSETS_GROUPS,
            controller: "document",
            index: engineId,
        });
        if (Array.isArray(result.errors) && result.errors.length > 0) {
            throw new kuzzle_1.BadRequestError(`The children group "${result.errors.join(",")}" does not exist`);
        }
    }
    async checkGroupName(engineId, name, assetId) {
        if (typeof name !== "string") {
            return;
        }
        const groupsCount = await this.sdk.document.count(engineId, plugin_1.InternalCollection.ASSETS_GROUPS, {
            query: {
                bool: {
                    must: [
                        {
                            regexp: {
                                name: {
                                    case_insensitive: true,
                                    value: name,
                                },
                            },
                        },
                    ],
                    must_not: [
                        {
                            terms: {
                                _id: typeof assetId === "string" ? [assetId] : [],
                            },
                        },
                    ],
                },
            },
        });
        if (groupsCount > 0) {
            throw new kuzzle_1.BadRequestError(`A group with name "${name}" already exist`);
        }
    }
    async create(request) {
        const engineId = request.getString("engineId");
        const name = request.getBodyString("name");
        const metadata = request.getBodyObject("metadata", {});
        const body = request.getBody();
        const model = body.model ?? null;
        const parent = body.parent ?? null;
        const _id = request.getId({
            generator: () => kuzzle_1.NameGenerator.generateRandomName({ prefix: "group" }),
            ifMissing: "generate",
        });
        await this.checkGroupName(engineId, name);
        if (parent !== null) {
            await this.checkParent(engineId, parent);
        }
        return this.assetsGroupsService.create(_id, engineId, metadata, model, name, parent, request);
    }
    async get(request) {
        const engineId = request.getString("engineId");
        const _id = request.getId();
        return this.assetsGroupsService.get(engineId, _id, request);
    }
    async update(request) {
        const engineId = request.getString("engineId");
        const _id = request.getId();
        const body = request.getBody();
        const name = body.name;
        const metadata = body.metadata;
        const children = body.children;
        const parent = body.parent;
        let updateRequestBody = {};
        if (parent !== undefined) {
            await this.checkParent(engineId, parent);
            updateRequestBody = { ...updateRequestBody, parent };
        }
        if (children !== undefined) {
            await this.checkChildren(engineId, children);
            updateRequestBody = { ...updateRequestBody, children };
        }
        if (name !== undefined) {
            await this.checkGroupName(engineId, name, _id);
            updateRequestBody = { ...updateRequestBody, name };
        }
        if (metadata !== undefined) {
            const group = await this.get(request);
            const { model, metadata: groupMetadata } = group._source;
            if (model !== null) {
                const groupModel = await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:model:group:get", { model });
                for (const metadataName of Object.keys(groupModel.group.metadataMappings)) {
                    if (metadata[metadataName] !== undefined) {
                        groupMetadata[metadataName] = metadata[metadataName];
                    }
                }
                updateRequestBody = { ...updateRequestBody, metadata: groupMetadata };
            }
        }
        updateRequestBody = { ...updateRequestBody, lastUpdate: Date.now() };
        return this.assetsGroupsService.update(request, _id, engineId, updateRequestBody);
    }
    async delete(request) {
        const engineId = request.getString("engineId");
        const _id = request.getId();
        await this.assetsGroupsService.delete(_id, engineId, request);
    }
    async search(request) {
        const engineId = request.getString("engineId");
        const searchParams = request.getSearchParams();
        return this.assetsGroupsService.search(engineId, searchParams, request);
    }
    async addAsset(request) {
        const engineId = request.getString("engineId");
        const _id = request.getId();
        const body = request.getBody();
        const assetIds = body.assetIds;
        return this.assetsGroupsService.addAsset(engineId, _id, assetIds, request);
    }
    async removeAsset(request) {
        const engineId = request.getString("engineId");
        const _id = request.getId();
        const body = request.getBody();
        return this.assetsGroupsService.removeAsset(engineId, _id, body.assetIds, request);
    }
}
exports.AssetsGroupsController = AssetsGroupsController;
//# sourceMappingURL=AssetsGroupsController.js.map