"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AssetsGroupsService = void 0;
const plugin_1 = require("../plugin");
const shared_1 = require("../shared");
const kuzzle_plugin_commons_1 = require("kuzzle-plugin-commons");
class AssetsGroupsService extends shared_1.BaseService {
    constructor(plugin) {
        super(plugin);
    }
    async create(_id, engineId, metadata, model, name, parent, request) {
        if (parent !== null) {
            const parentGroup = await this.sdk.document.get(engineId, plugin_1.InternalCollection.ASSETS_GROUPS, parent);
            const children = parentGroup._source.children ?? [];
            children.push(_id);
            await this.sdk.document.update(engineId, plugin_1.InternalCollection.ASSETS_GROUPS, parent, {
                children,
                lastUpdate: Date.now(),
            });
        }
        const groupMetadata = {};
        if (model !== null) {
            const groupModel = await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:model:group:get", { model });
            for (const metadataName of Object.keys(groupModel.group.metadataMappings)) {
                if (metadata[metadataName]) {
                    groupMetadata[metadataName] = metadata[metadataName];
                }
                else if (groupModel.group.defaultMetadata[metadataName]) {
                    groupMetadata[metadataName] =
                        groupModel.group.defaultMetadata[metadataName];
                }
                else {
                    groupMetadata[metadataName] = null;
                }
            }
        }
        const group = {
            _id,
            _source: {
                children: [],
                lastUpdate: Date.now(),
                metadata: { ...groupMetadata },
                model,
                name,
                parent,
            },
        };
        return this.createDocument(request, group, {
            collection: plugin_1.InternalCollection.ASSETS_GROUPS,
            engineId,
        });
    }
    async get(engineId, _id, request) {
        return this.getDocument(request, _id, {
            collection: plugin_1.InternalCollection.ASSETS_GROUPS,
            engineId,
        });
    }
    async update(request, _id, engineId, updateContent) {
        const updatedGroup = await this.updateDocument(request, {
            _id,
            _source: updateContent,
        }, { collection: plugin_1.InternalCollection.ASSETS_GROUPS, engineId }, { source: true, triggerEvents: true });
        return updatedGroup;
    }
    async delete(_id, engineId, request) {
        const { _source: assetGroup } = await this.get(engineId, _id, request);
        if (assetGroup.parent !== null) {
            const { _source: parentGroup } = await this.get(engineId, assetGroup.parent, request);
            await this.update(request, assetGroup.parent, engineId, {
                children: parentGroup.children.filter((children) => children !== _id),
                lastUpdate: Date.now(),
            });
        }
        await this.sdk.document.mUpdate(engineId, plugin_1.InternalCollection.ASSETS_GROUPS, assetGroup.children.map((childrenId) => ({
            _id: childrenId,
            body: {
                lastUpdate: Date.now(),
                parent: null,
            },
        })), { strict: true });
        const { hits: assets } = await this.sdk.document.search(engineId, plugin_1.InternalCollection.ASSETS, { query: { equals: { "groups.id": _id } } }, { lang: "koncorde" });
        await this.sdk.document.mUpdate(engineId, plugin_1.InternalCollection.ASSETS, assets.map((asset) => ({
            _id: asset._id,
            body: {
                groups: asset._source.groups.filter(({ id: groupId }) => groupId !== _id),
            },
        })), { strict: true });
        return this.deleteDocument(request, _id, {
            collection: plugin_1.InternalCollection.ASSETS_GROUPS,
            engineId,
        });
    }
    async search(engineId, searchParams, request) {
        return this.searchDocument(request, searchParams, {
            collection: plugin_1.InternalCollection.ASSETS_GROUPS,
            engineId,
        });
    }
    async addAsset(engineId, _id, assetIds, request) {
        const assetGroup = await this.get(engineId, _id, request);
        const assets = [];
        for (const assetId of assetIds) {
            const assetContent = (await this.getDocument(request, assetId, {
                collection: plugin_1.InternalCollection.ASSETS,
                engineId,
            }))._source;
            if (!Array.isArray(assetContent.groups)) {
                assetContent.groups = [];
            }
            if (assetGroup._source.parent !== null) {
                assetContent.groups.push({
                    date: Date.now(),
                    id: assetGroup._source.parent,
                });
            }
            assetContent.groups.push({
                date: Date.now(),
                id: _id,
            });
            assets.push({
                _id: assetId,
                body: assetContent,
            });
        }
        const assetsGroupsUpdate = await this.update(request, _id, engineId, {
            lastUpdate: Date.now(),
        });
        const update = await this.sdk.document.mReplace(engineId, plugin_1.InternalCollection.ASSETS, assets, { triggerEvents: true });
        return {
            ...update,
            assetsGroups: assetsGroupsUpdate,
        };
    }
    async removeAsset(engineId, _id, assetIds, request) {
        const { _source: AssetGroupContent } = await this.get(engineId, _id, request);
        const removedGroups = AssetGroupContent.children;
        removedGroups.push(_id);
        const assets = [];
        for (const assetId of assetIds) {
            const assetContent = (await this.getDocument(request, assetId, {
                collection: plugin_1.InternalCollection.ASSETS,
                engineId,
            }))._source;
            if (!Array.isArray(assetContent.groups)) {
                continue;
            }
            assetContent.groups = assetContent.groups.filter(({ id: groupId }) => !removedGroups.includes(groupId));
            assets.push({
                _id: assetId,
                body: assetContent,
            });
        }
        const assetsGroupsUpdate = await this.update(request, _id, engineId, {
            lastUpdate: Date.now(),
        });
        const update = await this.sdk.document.mReplace(engineId, plugin_1.InternalCollection.ASSETS, assets, { triggerEvents: true });
        return {
            ...update,
            assetsGroups: assetsGroupsUpdate,
        };
    }
}
exports.AssetsGroupsService = AssetsGroupsService;
//# sourceMappingURL=AssetsGroupsService.js.map