"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DecodedPayload = void 0;
const kuzzle_1 = require("kuzzle");
/**
 * Class containing the decoded measures.
 */
class DecodedPayload {
    constructor(decoder) {
        /**
         * Measurements per device.
         *
         * Record<deviceReference, Measurement[]>
         */
        this.measurementsByDevice = {};
        /**
         * Metadata per device.
         */
        this.metadataByDevice = {};
        this.decoder = decoder;
    }
    get references() {
        return Object.keys(this.measurementsByDevice);
    }
    /**
     * Add a new measure into the decoded payload
     *
     * @param deviceReference Device reference
     * @param measureName Name of the decoded measure
     * @param measurement Measurement object
     */
    addMeasurement(deviceReference, measureName, measurement) {
        if (!this.decoder.measureNames.includes(measureName)) {
            throw new kuzzle_1.BadRequestError(`Decoder "${this.decoder.deviceModel}" has no measure named "${measureName}"`);
        }
        this.validateMeasurement(measurement);
        if (!this.measurementsByDevice[deviceReference]) {
            this.measurementsByDevice[deviceReference] = [];
        }
        const decodedMeasurement = {
            measureName,
            ...measurement,
        };
        this.measurementsByDevice[deviceReference].push(decodedMeasurement);
    }
    /**
     * Add metadata values for a Device.
     *
     * Metadata should have been declared on the plugin to be used.
     *
     * @param deviceReference Device reference
     * @param metadata Object containing metadata values
     */
    addMetadata(deviceReference, metadata) {
        if (!this.metadataByDevice[deviceReference]) {
            this.metadataByDevice[deviceReference] = {};
        }
        this.metadataByDevice[deviceReference] = {
            ...this.metadataByDevice[deviceReference],
            ...metadata,
        };
    }
    /**
     * Gets the measurements decoded for a device
     */
    getMeasurements(deviceReference) {
        return this.measurementsByDevice[deviceReference];
    }
    /**
     * Get the metadata for a device
     */
    getMetadata(deviceReference) {
        return this.metadataByDevice[deviceReference];
    }
    validateMeasurement(measurement) {
        if (measurement.measuredAt.toString().length !== 13) {
            throw new kuzzle_1.BadRequestError(`Invalid payload: "measuredAt" should be a timestamp in milliseconds`);
        }
    }
}
exports.DecodedPayload = DecodedPayload;
//# sourceMappingURL=DecodedPayload.js.map