"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Decoder = void 0;
const kuzzle_1 = require("kuzzle");
const lodash_1 = __importDefault(require("lodash"));
/**
 * Base class to implement a decoder for a device model.
 * The device model must be passed to the parent constructor.
 * The abstract "decode" method must be implemented.
 */
class Decoder {
    constructor() {
        /**
         * Declaration of the measures decoded by this decoder.
         * The name correspond of the measure name for the device.
         * Measures types should be registered on the plugin beforehand.
         *
         * @example
         *
         * this.measures = [
         *   { type: 'temperature', name: 'temperatureExterior' },
         * ];
         */
        this.measures = [];
        /**
         * Custom mappings for the payload collection.
         * It will be injected in the "payload" property and it should allows to index
         * the device model unique identifier field.
         *
         * @example
         *
         * this.payloadsMappings = {
         *   device_properties: {
         *     properties: {
         *       deveui: { type: 'keyword' }
         *     }
         *   }
         * }
         */
        this.payloadsMappings = {};
    }
    /**
     * Define custom HTTP routes
     *
     * @param http HttpRoute array
     */
    set http(http) {
        this._http = http;
    }
    get http() {
        if (!this._http) {
            this._http = [
                {
                    openapi: {
                        description: `Receive a payload from a ${this.deviceModel} device`,
                        parameters: [
                            {
                                in: "body",
                                name: "payload",
                                required: true,
                                schema: {
                                    type: "object",
                                },
                            },
                        ],
                    },
                    path: `device-manager/payload/${this.action}`,
                    verb: "post",
                },
            ];
        }
        return this._http;
    }
    get measureNames() {
        return this.measures.map(({ name }) => name);
    }
    get measureTypes() {
        return this.measures.map(({ type }) => type);
    }
    /**
     * Validate the payload format before processing.
     *
     * If the method:
     *   - return true: the payload will be processed (status 200)
     *   - return false: the payload will be skipped (status 200)
     *   - throw an error: the payload will be skipped (status 4** or 5**)
     *
     * @param payload Raw payload received in the API action body
     * @param request Original request
     *
     * @return A boolean indicating if the payload is valid
     */
    // eslint-disable-next-line no-unused-vars
    async validate(payload, request) {
        return true;
    }
    /**
     * Checks if the provided properties are present in the payload
     *
     * @param payload Raw payload received in the API action body
     * @param paths Paths of properties (lodash style)
     *
     * @throws
     */
    ensureProperties(payload, paths) {
        for (const path of paths) {
            if (!lodash_1.default.has(payload, path)) {
                throw new kuzzle_1.PreconditionError(`Missing property "${path}" in payload`);
            }
        }
    }
    serialize() {
        return {
            action: this.action,
            deviceModel: this.deviceModel,
            measures: this.measures,
        };
    }
}
exports.Decoder = Decoder;
/* eslint-enable @typescript-eslint/no-unused-vars */
//# sourceMappingURL=Decoder.js.map