"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DecodersRegister = void 0;
const kuzzle_1 = require("kuzzle");
class DecodersRegister {
    constructor() {
        /**
         * Map<deviceModel, Decoder>
         */
        this._decoders = new Map();
    }
    get sdk() {
        return this.context.accessors.sdk;
    }
    get decoders() {
        return Array.from(this._decoders.values());
    }
    init(plugin, context) {
        this.plugin = plugin;
        this.context = context;
    }
    get(deviceModel) {
        if (!this._decoders.has(deviceModel)) {
            throw new kuzzle_1.BadRequestError(`Cannot find decoder for "${deviceModel}"`);
        }
        return this._decoders.get(deviceModel);
    }
    list() {
        const decoders = this.decoders.map((decoder) => decoder.serialize());
        return decoders;
    }
    /**
     * Registers a new decoder for a device model.
     * Decoder and device models share a 1:1 relationship
     *
     * This will register a new API action:
     *  - controller: `device-manager/payload`
     *  - action: `action` property of the decoder or the device model in kebab-case
     *
     * @param decoder Instantiated decoder
     *
     * @returns Corresponding API action requestPayload
     */
    register(decoder) {
        if (!decoder.deviceModel) {
            decoder.deviceModel = decoder.constructor.name.replace("Decoder", "");
        }
        decoder.action = decoder.action || kuzzle_1.Inflector.kebabCase(decoder.deviceModel);
        if (decoder.measures.length === 0) {
            throw new kuzzle_1.PluginImplementationError(`Decoder "${decoder.deviceModel}" did not declare any measures in the "decoder.measures" property.`);
        }
        if (this._decoders.has(decoder.deviceModel)) {
            throw new kuzzle_1.PluginImplementationError(`Decoder for device model "${decoder.deviceModel}" already registered`);
        }
        this._decoders.set(decoder.deviceModel, decoder);
        return {
            action: decoder.action,
            controller: "device-manager/payloads",
        };
    }
    printDecoders() {
        for (const decoder of this.decoders) {
            this.context.log.info(`Decoder for "${decoder.deviceModel}" registered`);
        }
    }
    /**
     * Register default role, profile and user associated to the generated actions
     * in the Payload controller.
     */
    registerDefaultRights() {
        if (this.decoders.length === 0) {
            return;
        }
        this.registerDefaultRole();
        this.registerDefaultProfile();
        this.registerDefaultUser();
    }
    registerDefaultUser() {
        const gatewayUser = {
            content: {
                profileIds: ["payload_gateway"],
            },
        };
        this.plugin.imports.users.payload_gateway = gatewayUser;
    }
    registerDefaultProfile() {
        const gatewayProfile = {
            policies: [{ roleId: "payload_gateway" }],
        };
        this.plugin.imports.profiles.payload_gateway = gatewayProfile;
    }
    registerDefaultRole() {
        const role = {
            controllers: {
                "device-manager/payloads": {
                    actions: {
                        "*": true,
                    },
                },
            },
        };
        this.plugin.imports.roles.payload_gateway = role;
    }
}
exports.DecodersRegister = DecodersRegister;
//# sourceMappingURL=DecodersRegister.js.map