"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PayloadService = void 0;
const kuzzle_1 = require("kuzzle");
const kuzzle_plugin_commons_1 = require("kuzzle-plugin-commons");
const uuid_1 = require("uuid");
const device_1 = require("../device");
const plugin_1 = require("../plugin");
const shared_1 = require("../shared");
const DecodedPayload_1 = require("./DecodedPayload");
const DecodingState_1 = require("./DecodingState");
const SkipError_1 = require("./SkipError");
class PayloadService extends shared_1.BaseService {
    constructor(plugin) {
        super(plugin);
        (0, kuzzle_plugin_commons_1.onAsk)("ask:device-manager:payload:receive-formated", async (payload) => {
            await this.receiveFormated(payload.device, payload.measures, {
                payloadUuids: payload.payloadUuids,
            });
        });
    }
    /**
     * Process a payload by validating and decode it with the `decoder` associated
     * with the device model. It :
     * - register the brut `Payload`
     * - redirect measurements to MeasureService
     */
    async receive(request, decoder, { refresh } = {}) {
        const payload = request.getBody();
        const apiAction = `${request.input.controller}:${request.input.action}`;
        const uuid = request.input.args.uuid || (0, uuid_1.v4)();
        let valid = true;
        let state = DecodingState_1.DecodingState.VALID;
        let errorReason;
        try {
            valid = await decoder.validate(payload, request);
            // TODO: Temporary workaround to prevent breaking anything; in the future,
            // consider modifying the return value of the 'validate' function to return an object
            if (!valid) {
                throw new SkipError_1.SkipError("Skip by user defined validation");
            }
        }
        catch (error) {
            valid = false;
            errorReason = error.message;
            if (error instanceof SkipError_1.SkipError) {
                state = DecodingState_1.DecodingState.SKIP;
                return { valid };
            }
            state = DecodingState_1.DecodingState.ERROR;
            throw error;
        }
        finally {
            await this.savePayload(decoder.deviceModel, uuid, valid, payload, apiAction, DecodingState_1.DecodingState[state], errorReason);
        }
        let decodedPayload = new DecodedPayload_1.DecodedPayload(decoder);
        decodedPayload = await decoder.decode(decodedPayload, payload, request);
        if (decodedPayload.references.length === 0) {
            throw new kuzzle_1.BadRequestError("No measurement has been decoded");
        }
        const devices = await this.retrieveDevices(decoder.deviceModel, decodedPayload.references, { refresh });
        for (const device of devices) {
            await (0, kuzzle_plugin_commons_1.ask)("device-manager:measures:ingest", {
                device,
                measurements: decodedPayload.getMeasurements(device._source.reference),
                metadata: decodedPayload.getMetadata(device._source.reference),
                payloadUuids: [uuid],
            });
        }
        return { valid };
    }
    async receiveFormated(device, measurements, { payloadUuids } = {}) {
        const apiAction = "device-manager/devices:receiveMeasure";
        // Payload is already formated thus valid
        await this.savePayload(device._source.model, payloadUuids[0], true, measurements, apiAction);
        await (0, kuzzle_plugin_commons_1.ask)("device-manager:measures:ingest", {
            device,
            measurements,
            metadata: {},
            payloadUuids,
        });
    }
    /**
     * Method used to save the payload and catch + log the error if any.
     *
     * This method never returns a rejected promise.
     */
    async savePayload(deviceModel, uuid, valid, payload, apiAction, state, reason) {
        try {
            await this.sdk.document.create(this.config.adminIndex, "payloads", { apiAction, deviceModel, payload, reason, state, uuid, valid }, uuid);
        }
        catch (error) {
            this.app.log.error(`Cannot save the payload from "${deviceModel}": ${error}`);
        }
    }
    /**
     * Get devices or create missing ones (when auto-provisionning is enabled)
     */
    async retrieveDevices(deviceModel, references, { refresh, } = {}) {
        const { successes: devices, errors } = await this.sdk.document.mGet(this.config.adminIndex, plugin_1.InternalCollection.DEVICES, references.map((reference) => device_1.DeviceSerializer.id(deviceModel, reference)));
        // Due to the existence of a "devices" collection in the tenant index and a platform index,
        // we need to fetch the device content from the associated tenant if it exists.
        const updatedDevices = await Promise.all(devices.map((device) => device._source.engineId && device._source.engineId.trim() !== ""
            ? this.sdk.document.get(device._source.engineId, plugin_1.InternalCollection.DEVICES, device._id)
            : device));
        // If we have unknown devices, let's check if we should register them
        if (errors.length > 0) {
            const { _source } = await this.sdk.document.get(this.config.adminIndex, this.config.adminCollections.config.name, "plugin--device-manager");
            if (_source["device-manager"].provisioningStrategy === "auto") {
                const newDevices = await this.provisionDevices(deviceModel, errors, {
                    refresh,
                });
                updatedDevices.push(...newDevices);
            }
            else {
                this.app.log.info(`Skipping new devices "${errors.join(", ")}". Auto-provisioning is disabled.`);
            }
        }
        return updatedDevices;
    }
    async provisionDevices(deviceModel, deviceIds, { refresh }) {
        const newDevices = deviceIds.map((deviceId) => {
            // Reference may contains a "-"
            const [, ...rest] = deviceId.split("-");
            const reference = rest.join("-");
            const body = {
                assetId: null,
                engineId: null,
                lastMeasuredAt: 0,
                measures: {},
                metadata: {},
                model: deviceModel,
                reference,
            };
            return {
                _id: device_1.DeviceSerializer.id(deviceModel, reference),
                body,
            };
        });
        const { successes, errors } = await this.sdk.document.mCreate(this.config.adminIndex, plugin_1.InternalCollection.DEVICES, newDevices, { refresh });
        for (const error of errors) {
            this.app.log.error(`Cannot create device "${error.document._id}": ${error.reason}`);
        }
        return successes;
    }
    async prune(days, onlyValid, { deviceModel } = {}) {
        const filter = [];
        const date = new Date().setDate(new Date().getDate() - days);
        filter.push({
            range: {
                "_kuzzle_info.createdAt": {
                    lt: date,
                },
            },
        });
        if (onlyValid === true) {
            filter.push({ term: { valid: true } });
        }
        if (deviceModel) {
            filter.push({ term: { deviceModel } });
        }
        const deleted = await this.sdk.bulk.deleteByQuery(this.config.adminIndex, "payloads", { query: { bool: { filter } } });
        return deleted;
    }
    async receiveUnknown(deviceModel, payload, apiAction) {
        await this.savePayload(deviceModel, (0, uuid_1.v4)(), false, payload, apiAction);
    }
}
exports.PayloadService = PayloadService;
//# sourceMappingURL=PayloadService.js.map