"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeviceService = void 0;
const kuzzle_1 = require("kuzzle");
const kuzzle_plugin_commons_1 = require("kuzzle-plugin-commons");
const plugin_1 = require("../plugin");
const shared_1 = require("../shared");
const DeviceSerializer_1 = require("./model/DeviceSerializer");
const lodash_1 = __importDefault(require("lodash"));
class DeviceService extends shared_1.DigitalTwinService {
    constructor(plugin) {
        super(plugin, plugin_1.InternalCollection.DEVICES);
    }
    registerAskEvents() {
        super.registerAskEvents();
        (0, kuzzle_plugin_commons_1.onAsk)("ask:device-manager:device:link-asset", async ({ deviceId, engineId, user, assetId, measureNames }) => {
            const request = new kuzzle_1.KuzzleRequest({ refresh: "false" }, { user });
            await this.linkAsset(engineId, deviceId, assetId, measureNames, false, request);
        });
        (0, kuzzle_plugin_commons_1.onAsk)("ask:device-manager:device:unlink-asset", async ({ deviceId, user }) => {
            const request = new kuzzle_1.KuzzleRequest({ refresh: "false" }, { user });
            await this.unlinkAsset(deviceId, request);
        });
        (0, kuzzle_plugin_commons_1.onAsk)("ask:device-manager:device:detach-engine", async ({ deviceId, user }) => {
            const request = new kuzzle_1.KuzzleRequest({ refresh: "false" }, { user });
            await this.detachEngine(deviceId, request);
        });
        (0, kuzzle_plugin_commons_1.onAsk)("ask:device-manager:device:attach-engine", async ({ deviceId, engineId, user }) => {
            const request = new kuzzle_1.KuzzleRequest({ refresh: "false" }, { user });
            await this.attachEngine(engineId, deviceId, request);
        });
    }
    async _create(deviceId, model, reference, metadata, request) {
        var _a;
        let device = {
            _id: deviceId,
            _source: {
                assetId: null,
                engineId: null,
                lastMeasuredAt: 0,
                measures: {},
                metadata,
                model,
                reference,
            },
        };
        const deviceModel = await this.getDeviceModel(model);
        const engineId = request.getString("engineId");
        for (const metadataName of Object.keys(deviceModel.device.metadataMappings)) {
            (_a = device._source.metadata)[metadataName] || (_a[metadataName] = null);
        }
        for (const [metadataName, metadataValue] of Object.entries(deviceModel.device.defaultMetadata)) {
            lodash_1.default.set(device._source.metadata, metadataName, metadataValue);
        }
        const refreshableCollections = [];
        const { _source } = await this.createDocument(request, device, {
            collection: plugin_1.InternalCollection.DEVICES,
            engineId: this.config.adminIndex,
        });
        device._source = _source;
        refreshableCollections.push({
            collection: plugin_1.InternalCollection.DEVICES,
            index: this.config.adminIndex,
        });
        if (engineId && engineId !== this.config.adminIndex) {
            device = await this._attachEngine(engineId, device._id, request);
            refreshableCollections.push({
                collection: plugin_1.InternalCollection.DEVICES,
                index: engineId,
            });
        }
        if (request.getRefresh() === "wait_for") {
            await Promise.all(refreshableCollections.map(({ index, collection }) => this.sdk.collection.refresh(index, collection)));
        }
        return device;
    }
    /**
     * Create a new device.
     *
     * @todo creating a device in the "device-manager" index should be a separate
     * method since in this case "engineId" is not really an engine ID but still
     * required as an argument (in part to check the tenant rights)
     */
    async create(model, reference, metadata, request) {
        const deviceId = DeviceSerializer_1.DeviceSerializer.id(model, reference);
        return (0, shared_1.lock)(`device:${deviceId}`, async () => this._create(deviceId, model, reference, metadata, request));
    }
    async get(engineId, deviceId, request) {
        return this.getDocument(request, deviceId, {
            collection: plugin_1.InternalCollection.DEVICES,
            engineId,
        });
    }
    async getInternalDevices(deviceId) {
        return this.sdk.document.get(this.config.adminIndex, plugin_1.InternalCollection.DEVICES, deviceId);
    }
    /**
     * Replace a device metadata
     */
    async replaceMetadata(engineId, deviceId, metadata, request) {
        return (0, shared_1.lock)(`device:${deviceId}`, async () => {
            const device = await this.get(engineId, deviceId, request);
            for (const key in metadata) {
                if (key in device._source.metadata) {
                    device._source.metadata[key] = metadata[key];
                }
            }
            const updatedPayload = await this.app.trigger("device-manager:device:update:before", { device: device, metadata });
            const updatedDevice = await this.sdk.document.replace(engineId, plugin_1.InternalCollection.DEVICES, deviceId, updatedPayload.device._source);
            await this.app.trigger("device-manager:device:update:after", {
                device: updatedDevice,
                metadata: updatedPayload.metadata,
            });
            return updatedDevice;
        });
    }
    /**
     * Update or Create an device metadata
     */
    async upsert(engineId, model, reference, metadata, request) {
        const deviceId = DeviceSerializer_1.DeviceSerializer.id(model, reference);
        return (0, shared_1.lock)(`device:${deviceId}`, async () => {
            const adminIndexDevice = await this.get(this.config.adminIndex, deviceId, request).catch(() => null);
            if (!adminIndexDevice) {
                return this._create(deviceId, model, reference, metadata, request);
            }
            if (adminIndexDevice._source.engineId &&
                adminIndexDevice._source.engineId !== engineId) {
                throw new kuzzle_1.BadRequestError(`Device "${adminIndexDevice._id}" already exists on another engine. Abort`);
            }
            const engineDevice = await this.get(engineId, deviceId, request).catch(() => null);
            if (!engineDevice) {
                await this._attachEngine(engineId, deviceId, request);
            }
            const updatedPayload = await this.app.trigger("device-manager:device:update:before", { device: adminIndexDevice, metadata });
            const updatedDevice = await this.updateDocument(request, {
                _id: deviceId,
                _source: { metadata: updatedPayload.metadata },
            }, {
                collection: plugin_1.InternalCollection.DEVICES,
                engineId,
            }, { source: true });
            await this.app.trigger("device-manager:device:update:after", {
                device: updatedDevice,
                metadata: updatedPayload.metadata,
            });
            return updatedDevice;
        });
    }
    async update(engineId, deviceId, metadata, request) {
        return (0, shared_1.lock)(`device:${deviceId}`, async () => {
            const device = await this.get(engineId, deviceId, request);
            const updatedPayload = await this.app.trigger("device-manager:device:update:before", { device, metadata });
            const updatedDevice = await this.updateDocument(request, {
                _id: deviceId,
                _source: { metadata: updatedPayload.metadata },
            }, { collection: plugin_1.InternalCollection.DEVICES, engineId }, { source: true });
            await this.app.trigger("device-manager:device:update:after", {
                device: updatedDevice,
                metadata: updatedPayload.metadata,
            });
            return updatedDevice;
        });
    }
    async delete(engineId, deviceId, request) {
        return (0, shared_1.lock)(`device:${deviceId}`, async () => {
            const device = await this.get(engineId, deviceId, request);
            const promises = [];
            if (device._source.assetId) {
                const asset = await this.sdk.document.get(engineId, plugin_1.InternalCollection.ASSETS, device._source.assetId);
                const linkedDevices = asset._source.linkedDevices.filter((link) => link._id !== device._id);
                promises.push(
                // Potential race condition if someone update the asset linkedDevices
                // at the same time (e.g link or unlink asset)
                this.updateDocument(request, {
                    _id: device._source.assetId,
                    _source: { linkedDevices },
                }, { collection: plugin_1.InternalCollection.ASSETS, engineId }).then(async (updatedAsset) => {
                    const event = {
                        name: "unlink",
                        unlink: {
                            deviceId,
                        },
                    };
                    await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:asset:history:add", {
                        engineId,
                        histories: [
                            {
                                asset: updatedAsset._source,
                                event,
                                id: updatedAsset._id,
                                timestamp: Date.now(),
                            },
                        ],
                    });
                }));
            }
            promises.push(this.deleteDocument(request, deviceId, {
                collection: plugin_1.InternalCollection.DEVICES,
                engineId: this.config.adminIndex,
            }));
            promises.push(this.deleteDocument(request, deviceId, {
                collection: plugin_1.InternalCollection.DEVICES,
                engineId,
            }));
            await Promise.all(promises);
        });
    }
    async search(engineId, searchParams, request) {
        return this.searchDocument(request, searchParams, {
            collection: plugin_1.InternalCollection.DEVICES,
            engineId,
        });
    }
    /**
     * Internal logic to attach the device to an engine
     *
     * @param engineId Engine id to attach to
     * @param deviceId Device id to attach
     * @param options.refresh Wait for ES indexation
     * @param options.strict If true, throw if an operation isn't possible
     */
    async _attachEngine(engineId, deviceId, request) {
        const device = await this.getInternalDevices(deviceId);
        if (device._source.engineId) {
            throw new kuzzle_1.BadRequestError(`Device "${device._id}" is already attached to an engine.`);
        }
        await this.checkEngineExists(engineId);
        device._source.engineId = engineId;
        await this.updateDocument(request, device, {
            collection: plugin_1.InternalCollection.DEVICES,
            engineId: this.config.adminIndex,
        });
        // Make sure the device is cleaned when attached to tenant
        device._source.lastMeasuredAt = null;
        device._source.measures = {};
        const updatedDevice = await this.createDocument(request, device, {
            collection: plugin_1.InternalCollection.DEVICES,
            engineId,
        });
        if (request.getRefresh() === "wait_for") {
            await Promise.all([
                this.sdk.collection.refresh(this.config.adminIndex, plugin_1.InternalCollection.DEVICES),
                this.sdk.collection.refresh(device._source.engineId, plugin_1.InternalCollection.DEVICES),
            ]);
        }
        return updatedDevice;
    }
    /**
     * Attach the device to an engine
     *
     * @param engineId Engine id to attach to
     * @param deviceId Device id to attach
     * @param options.refresh Wait for ES indexation
     * @param options.strict If true, throw if an operation isn't possible
     */
    async attachEngine(engineId, deviceId, request) {
        return (0, shared_1.lock)(`device:${deviceId}`, async () => this._attachEngine(engineId, deviceId, request));
    }
    /**
     * Detach a device from its attached engine
     *
     * @param {string} deviceId Id of the device
     * @param {KuzzleRequest} request kuzzle request
     */
    async detachEngine(deviceId, request) {
        return (0, shared_1.lock)(`device:${deviceId}`, async () => {
            const device = await this.getInternalDevices(deviceId);
            this.checkAttachedToEngine(device);
            if (device._source.assetId) {
                await this.unlinkAsset(deviceId, request);
            }
            await Promise.all([
                this.updateDocument(request, {
                    _id: device._id,
                    _source: { engineId: null },
                }, {
                    collection: plugin_1.InternalCollection.DEVICES,
                    engineId: this.config.adminIndex,
                }),
                this.sdk.document.delete(device._source.engineId, plugin_1.InternalCollection.DEVICES, device._id),
            ]);
            if (request.getRefresh() === "wait_for") {
                await Promise.all([
                    this.sdk.collection.refresh(this.config.adminIndex, plugin_1.InternalCollection.DEVICES),
                    this.sdk.collection.refresh(device._source.engineId, plugin_1.InternalCollection.DEVICES),
                ]);
            }
            return device;
        });
    }
    /**
     * Link a device to an asset.
     */
    async linkAsset(engineId, deviceId, assetId, measureNames, implicitMeasuresLinking, request) {
        return (0, shared_1.lock)(`device:${deviceId}`, async () => {
            const device = await this.getInternalDevices(deviceId);
            const engine = await this.getEngine(engineId);
            this.checkAttachedToEngine(device);
            if (device._source.engineId !== engineId) {
                throw new kuzzle_1.BadRequestError(`Device "${device._id}" is not attached to the specified engine.`);
            }
            if (device._source.assetId && device._source.assetId !== assetId) {
                throw new kuzzle_1.BadRequestError(`Device "${device._id}" is already linked to another asset.`);
            }
            const asset = await this.sdk.document.get(device._source.engineId, plugin_1.InternalCollection.ASSETS, assetId);
            // Remove existing links for this device
            asset._source.linkedDevices = asset._source.linkedDevices.filter((link) => {
                return link._id !== deviceId;
            });
            const [assetModel, deviceModel] = await Promise.all([
                this.getAssetModel(engine.group, asset._source.model),
                this.getDeviceModel(device._source.model),
            ]);
            const updatedMeasureNames = [];
            for (const measure of measureNames) {
                const foundMeasure = deviceModel.device.measures.find((deviceMeasure) => deviceMeasure.name === measure.device);
                if (!foundMeasure && !implicitMeasuresLinking) {
                    throw new kuzzle_1.BadRequestError(`Measure "${measure.asset}" is not declared in the device model "${measure.device}".`);
                }
                const { type } = foundMeasure;
                updatedMeasureNames.push({
                    asset: measure.asset,
                    device: measure.device,
                    type,
                });
            }
            this.checkAlreadyProvidedMeasures(asset, updatedMeasureNames);
            if (implicitMeasuresLinking) {
                this.generateMissingAssetMeasureNames(asset, assetModel, deviceModel, updatedMeasureNames);
            }
            device._source.assetId = assetId;
            asset._source.linkedDevices.push({
                _id: deviceId,
                measureNames: updatedMeasureNames,
            });
            const [updatedDevice, , updatedAsset] = await Promise.all([
                this.updateDocument(request, device, {
                    collection: plugin_1.InternalCollection.DEVICES,
                    engineId: this.config.adminIndex,
                }, { source: true }),
                this.updateDocument(request, device, {
                    collection: plugin_1.InternalCollection.DEVICES,
                    engineId: device._source.engineId,
                }),
                this.updateDocument(request, asset, {
                    collection: plugin_1.InternalCollection.ASSETS,
                    engineId: device._source.engineId,
                }, { source: true }),
            ]);
            const event = {
                link: {
                    deviceId: device._id,
                },
                name: "link",
            };
            await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:asset:history:add", {
                engineId,
                histories: [
                    {
                        asset: updatedAsset._source,
                        event,
                        id: updatedAsset._id,
                        timestamp: Date.now(),
                    },
                ],
            });
            if (request.getRefresh() === "wait_for") {
                await Promise.all([
                    this.sdk.collection.refresh(this.config.adminIndex, plugin_1.InternalCollection.DEVICES),
                    this.sdk.collection.refresh(device._source.engineId, plugin_1.InternalCollection.DEVICES),
                    this.sdk.collection.refresh(device._source.engineId, plugin_1.InternalCollection.ASSETS),
                ]);
            }
            return { asset: updatedAsset, device: updatedDevice };
        });
    }
    async receiveMeasures(engineId, deviceId, measures, payloadUuids, request) {
        const device = await this.get(engineId, deviceId, request);
        const deviceModel = await this.getDeviceModel(device._source.model);
        for (const measure of measures) {
            const declaredMeasure = deviceModel.device.measures.some((m) => m.name === measure.measureName && m.type === measure.type);
            if (!declaredMeasure) {
                throw new kuzzle_1.BadRequestError(`Measure "${measure.measureName}" is not declared for this device model.`);
            }
        }
        await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:payload:receive-formated", {
            device,
            measures,
            payloadUuids,
        });
    }
    /**
     * Checks if the asset does not already have a linked device using one of the
     * requested measure names.
     */
    checkAlreadyProvidedMeasures(asset, requestedMeasureNames) {
        const measureAlreadyProvided = (assetMeasureName) => {
            return asset._source.linkedDevices.some((link) => link.measureNames.some((names) => names.asset === assetMeasureName));
        };
        for (const name of requestedMeasureNames) {
            if (measureAlreadyProvided(name.asset)) {
                throw new kuzzle_1.BadRequestError(`Measure name "${name.asset}" is already provided by another device on this asset.`);
            }
        }
    }
    /**
     * Goes through the device available measures and add them into the link if:
     *  - they are not already provided by another device
     *  - they are not already present in the link request
     *  - they are declared in the asset model
     */
    generateMissingAssetMeasureNames(asset, assetModel, deviceModel, requestedMeasureNames) {
        const measureAlreadyProvided = (deviceMeasureName) => {
            return asset._source.linkedDevices.some((link) => link.measureNames.some((names) => names.device === deviceMeasureName));
        };
        const measureAlreadyRequested = (deviceMeasureName) => {
            return requestedMeasureNames.some((names) => names.device === deviceMeasureName);
        };
        const measureUndeclared = (deviceMeasureName) => {
            return !assetModel.asset.measures.some((measure) => measure.name === deviceMeasureName);
        };
        for (const deviceMeasure of deviceModel.device.measures) {
            if (measureAlreadyRequested(deviceMeasure.name) ||
                measureAlreadyProvided(deviceMeasure.name) ||
                measureUndeclared(deviceMeasure.name)) {
                continue;
            }
            requestedMeasureNames.push({
                asset: deviceMeasure.name,
                device: deviceMeasure.name,
                type: deviceMeasure.type,
            });
        }
    }
    /**
     * Internal logic of unlink a device of an asset
     *
     * @param {string} deviceId Id of the device
     * @param {KuzzleRequest} request kuzzle request
     */
    async _unlinkAsset(deviceId, request) {
        const device = await this.getInternalDevices(deviceId);
        const engineId = device._source.engineId;
        this.checkAttachedToEngine(device);
        if (!device._source.assetId) {
            throw new kuzzle_1.BadRequestError(`Device "${device._id}" is not linked to an asset.`);
        }
        const asset = await this.sdk.document.get(engineId, plugin_1.InternalCollection.ASSETS, device._source.assetId);
        const linkedDevices = asset._source.linkedDevices.filter((link) => link._id !== device._id);
        const [updatedDevice, , updatedAsset] = await Promise.all([
            this.updateDocument(request, { _id: device._id, _source: { assetId: null } }, {
                collection: plugin_1.InternalCollection.DEVICES,
                engineId: this.config.adminIndex,
            }, { source: true }),
            this.updateDocument(request, { _id: device._id, _source: { assetId: null } }, {
                collection: plugin_1.InternalCollection.DEVICES,
                engineId,
            }),
            this.updateDocument(request, { _id: asset._id, _source: { linkedDevices } }, {
                collection: plugin_1.InternalCollection.ASSETS,
                engineId,
            }, { source: true }),
        ]);
        const event = {
            name: "unlink",
            unlink: {
                deviceId,
            },
        };
        await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:asset:history:add", {
            engineId,
            histories: [
                {
                    asset: updatedAsset._source,
                    event,
                    id: updatedAsset._id,
                    timestamp: Date.now(),
                },
            ],
        });
        if (request.getRefresh() === "wait_for") {
            await Promise.all([
                this.sdk.collection.refresh(this.config.adminIndex, plugin_1.InternalCollection.DEVICES),
                this.sdk.collection.refresh(engineId, plugin_1.InternalCollection.DEVICES),
                this.sdk.collection.refresh(engineId, plugin_1.InternalCollection.ASSETS),
            ]);
        }
        return { asset: updatedAsset, device: updatedDevice };
    }
    /**
     * Unlink a device of an asset with lock
     *
     * @param {string} deviceId Id of the device
     * @param {KuzzleRequest} request kuzzle request
     */
    async unlinkAsset(deviceId, request) {
        return (0, shared_1.lock)(`device:${deviceId}`, async () => this._unlinkAsset(deviceId, request));
    }
    async checkEngineExists(engineId) {
        const { result: { exists }, } = await this.sdk.query({
            action: "exists",
            controller: "device-manager/engine",
            index: engineId,
        });
        if (!exists) {
            throw new kuzzle_1.BadRequestError(`Engine "${engineId}" does not exists.`);
        }
    }
    checkAttachedToEngine(device) {
        if (!device._source.engineId) {
            throw new kuzzle_1.BadRequestError(`Device "${device._id}" is not attached to an engine.`);
        }
    }
    async getEngine(engineId) {
        const engine = await this.sdk.document.get(this.config.adminIndex, plugin_1.InternalCollection.CONFIG, `engine-device-manager--${engineId}`);
        return engine._source.engine;
    }
    getDeviceModel(model) {
        return (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:model:device:get", {
            model,
        });
    }
    getAssetModel(engineGroup, model) {
        return (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:model:asset:get", {
            engineGroup,
            model,
        });
    }
}
exports.DeviceService = DeviceService;
//# sourceMappingURL=DeviceService.js.map