"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DevicesController = void 0;
const kuzzle_1 = require("kuzzle");
const lodash_1 = __importDefault(require("lodash"));
const AssetSerializer_1 = require("../asset/model/AssetSerializer");
const measure_1 = require("../measure");
const plugin_1 = require("../plugin");
const shared_1 = require("../shared");
const DeviceSerializer_1 = require("./model/DeviceSerializer");
class DevicesController {
    constructor(plugin, deviceService) {
        this.plugin = plugin;
        this.deviceService = deviceService;
        /* eslint-disable sort-keys */
        this.definition = {
            actions: {
                create: {
                    handler: this.create.bind(this),
                    http: [{ path: "device-manager/:engineId/devices", verb: "post" }],
                },
                get: {
                    handler: this.get.bind(this),
                    http: [
                        { path: "device-manager/:engineId/devices/:_id", verb: "get" },
                    ],
                },
                update: {
                    handler: this.update.bind(this),
                    http: [
                        { path: "device-manager/:engineId/devices/:_id", verb: "put" },
                    ],
                },
                upsert: {
                    handler: this.upsert.bind(this),
                    http: [{ path: "device-manager/:engineId/devices", verb: "put" }],
                },
                replaceMetadata: {
                    handler: this.replaceMetadata.bind(this),
                    http: [
                        {
                            path: "device-manager/:engineId/devices/:_id/metadata",
                            verb: "patch",
                        },
                    ],
                },
                search: {
                    handler: this.search.bind(this),
                    http: [
                        { path: "device-manager/:engineId/devices/_search", verb: "post" },
                    ],
                },
                delete: {
                    handler: this.delete.bind(this),
                    http: [
                        {
                            path: "device-manager/:engineId/devices/:_id",
                            verb: "delete",
                        },
                    ],
                },
                attachEngine: {
                    handler: this.attachEngine.bind(this),
                    http: [
                        {
                            path: "device-manager/:engineId/devices/:_id/_attach",
                            verb: "put",
                        },
                    ],
                },
                detachEngine: {
                    handler: this.detachEngine.bind(this),
                    http: [
                        { path: "device-manager/devices/:_id/_detach", verb: "delete" },
                    ],
                },
                linkAsset: {
                    handler: this.linkAsset.bind(this),
                    http: [
                        {
                            path: "device-manager/:engineId/devices/:_id/_link/:assetId",
                            verb: "put",
                        },
                    ],
                },
                unlinkAsset: {
                    handler: this.unlinkAsset.bind(this),
                    http: [
                        {
                            path: "device-manager/:engineId/devices/:_id/_unlink",
                            verb: "delete",
                        },
                    ],
                },
                getMeasures: {
                    handler: this.getMeasures.bind(this),
                    http: [
                        {
                            path: "device-manager/:engineId/devices/:_id/measures",
                            verb: "get",
                        },
                        {
                            path: "device-manager/:engineId/devices/:_id/measures",
                            verb: "post",
                        },
                    ],
                },
                getLastMeasures: {
                    handler: this.getLastMeasures.bind(this),
                    http: [
                        {
                            path: "device-manager/:engineId/devices/:_id/lastMeasures",
                            verb: "get",
                        },
                        {
                            path: "device-manager/:engineId/devices/:_id/lastMeasures",
                            verb: "post",
                        },
                    ],
                },
                mGetLastMeasures: {
                    handler: this.mGetLastMeasures.bind(this),
                    http: [
                        {
                            path: "device-manager/:engineId/devices/_mGetLastMeasures",
                            verb: "post",
                        },
                    ],
                },
                exportMeasures: {
                    handler: this.exportMeasures.bind(this),
                    http: [
                        {
                            path: "device-manager/:engineId/devices/:_id/measures/_export/:exportId",
                            verb: "get",
                        },
                        {
                            path: "device-manager/:engineId/devices/:_id/measures/_export",
                            verb: "post",
                        },
                    ],
                },
                receiveMeasures: {
                    handler: this.receiveMeasures.bind(this),
                    http: [
                        {
                            path: "device-manager/:engineId/devices/:_id/measures",
                            verb: "put",
                        },
                    ],
                },
                export: {
                    handler: this.export.bind(this),
                    http: [
                        {
                            path: "device-manager/:engineId/devices/_export/:exportId",
                            verb: "get",
                        },
                        {
                            path: "device-manager/:engineId/devices/_export",
                            verb: "post",
                        },
                    ],
                },
                getLastMeasuredAt: {
                    handler: this.getLastMeasuredAt.bind(this),
                    http: [
                        {
                            path: "device-manager/:engineId/devices/:_id/lastMeasuredAt",
                            verb: "get",
                        },
                        {
                            path: "device-manager/:engineId/devices/:_id/lastMeasuredAt",
                            verb: "post",
                        },
                    ],
                },
                mGetLastMeasuredAt: {
                    handler: this.mGetLastMeasuredAt.bind(this),
                    http: [
                        {
                            path: "device-manager/:engineId/devices/_mGetLastMeasuredAt",
                            verb: "post",
                        },
                    ],
                },
            },
        };
        /* eslint-enable sort-keys */
        this.exporter = new shared_1.DigitalTwinExporter(this.plugin, plugin_1.InternalCollection.DEVICES);
        this.measureExporter = new measure_1.MeasureExporter(this.plugin, plugin_1.InternalCollection.DEVICES);
    }
    async get(request) {
        const deviceId = request.getId();
        const engineId = request.getString("engineId");
        const device = await this.deviceService.get(engineId, deviceId, request);
        return DeviceSerializer_1.DeviceSerializer.serialize(device);
    }
    async update(request) {
        const deviceId = request.getId();
        const engineId = request.getString("engineId");
        const metadata = request.getBodyObject("metadata");
        const updatedDevice = await this.deviceService.update(engineId, deviceId, metadata, request);
        return DeviceSerializer_1.DeviceSerializer.serialize(updatedDevice);
    }
    async replaceMetadata(request) {
        const deviceId = request.getId();
        const engineId = request.getString("engineId");
        const metadata = request.getBodyObject("metadata");
        const updatedDevice = await this.deviceService.replaceMetadata(engineId, deviceId, metadata, request);
        return DeviceSerializer_1.DeviceSerializer.serialize(updatedDevice);
    }
    async delete(request) {
        const engineId = request.getString("engineId");
        const deviceId = request.getId();
        await this.deviceService.delete(engineId, deviceId, request);
    }
    async search(request) {
        return this.deviceService.search(request.getString("engineId"), request.getSearchParams(), request);
    }
    async upsert(request) {
        const engineId = request.getString("engineId");
        const model = request.getBodyString("model");
        const reference = request.getBodyString("reference");
        const metadata = request.getBodyObject("metadata");
        const upsertDevice = await this.deviceService.upsert(engineId, model, reference, metadata, request);
        return DeviceSerializer_1.DeviceSerializer.serialize(upsertDevice);
    }
    /**
     * Create and provision a new device
     */
    async create(request) {
        const model = request.getBodyString("model");
        const reference = request.getBodyString("reference");
        const metadata = request.getBodyObject("metadata", {});
        const device = await this.deviceService.create(model, reference, metadata, request);
        return DeviceSerializer_1.DeviceSerializer.serialize(device);
    }
    /**
     * Attach a device to a tenant
     */
    async attachEngine(request) {
        const engineId = request.getString("engineId");
        const deviceId = request.getId();
        await this.deviceService.attachEngine(engineId, deviceId, request);
    }
    /**
     * Detach a device from it's tenant
     */
    async detachEngine(request) {
        const deviceId = request.getId();
        await this.deviceService.detachEngine(deviceId, request);
    }
    /**
     * Link a device to an asset.
     */
    async linkAsset(request) {
        const deviceId = request.getId();
        const engineId = request.getString("engineId");
        const assetId = request.getString("assetId");
        const measureNames = request.getBodyArray("measureNames", []);
        const implicitMeasuresLinking = request.getBoolean("implicitMeasuresLinking");
        if (measureNames.length === 0 && !implicitMeasuresLinking) {
            throw new kuzzle_1.BadRequestError(`You must provide at least one measure name or set "implicitMeasuresLinking" to true.`);
        }
        const { asset, device } = await this.deviceService.linkAsset(engineId, deviceId, assetId, measureNames, implicitMeasuresLinking, request);
        return {
            asset: AssetSerializer_1.AssetSerializer.serialize(asset),
            device: DeviceSerializer_1.DeviceSerializer.serialize(device),
        };
    }
    /**
     * Unlink a device from an asset.
     */
    async unlinkAsset(request) {
        const deviceId = request.getId();
        const { asset, device } = await this.deviceService.unlinkAsset(deviceId, request);
        return {
            asset: AssetSerializer_1.AssetSerializer.serialize(asset),
            device: DeviceSerializer_1.DeviceSerializer.serialize(device),
        };
    }
    async getMeasures(request) {
        const id = request.getId();
        const engineId = request.getString("engineId");
        const size = request.input.args.size;
        const from = request.input.args.from;
        const startAt = request.input.args.startAt
            ? request.getDate("startAt")
            : null;
        const endAt = request.input.args.endAt ? request.getDate("endAt") : null;
        const query = request.input.body?.query;
        const sort = request.input.body?.sort;
        const type = request.input.args.type;
        const lang = request.getLangParam();
        const { measures, total } = await this.measureExporter.search(engineId, {
            endAt,
            id,
            lang,
            query,
            sort,
            startAt,
            type,
        }, { from, size });
        return { measures, total };
    }
    async getLastMeasures(request) {
        const deviceId = request.getId();
        const engineId = request.getString("engineId");
        const measureCount = request.getNumber("measureCount", 100);
        const results = await this.deviceService.getLastMeasures(engineId, deviceId, measureCount);
        return results.reduce((accumulator, result) => {
            if (result.origin.type !== "device") {
                return accumulator;
            }
            const measure = {
                measuredAt: result.measuredAt,
                name: result.origin.measureName,
                originId: result.origin._id,
                payloadUuids: result.origin.payloadUuids,
                type: result.type,
                values: result.values,
            };
            return {
                ...accumulator,
                [result.origin.measureName]: measure,
            };
        }, {});
    }
    async mGetLastMeasures(request) {
        const engineId = request.getString("engineId");
        const measureCount = request.getNumber("measureCount", 100);
        const deviceIds = request.getBodyArray("ids");
        const results = await this.deviceService.mGetLastMeasures(engineId, deviceIds, measureCount);
        const response = {};
        for (const [deviceId, measures] of Object.entries(results)) {
            response[deviceId] = measures.reduce((accumulator, result) => {
                if (result.origin.type !== "device") {
                    return accumulator;
                }
                const measure = {
                    measuredAt: result.measuredAt,
                    name: result.origin.measureName,
                    originId: result.origin._id,
                    payloadUuids: result.origin.payloadUuids,
                    type: result.type,
                    values: result.values,
                };
                return {
                    ...accumulator,
                    [result.origin.measureName]: measure,
                };
            }, {});
        }
        return response;
    }
    async exportMeasures(request) {
        const engineId = request.getString("engineId");
        if (request.context.connection.protocol === "http" &&
            request.context.connection.misc.verb === "GET") {
            try {
                const exportId = request.getString("exportId");
                const { id } = await this.measureExporter.getExport(engineId, exportId);
                const stream = await this.measureExporter.sendExport(engineId, exportId);
                request.response.configure({
                    headers: {
                        "Content-Disposition": `attachment; filename="device-${id}.csv"`,
                        "Content-Type": "text/csv",
                    },
                });
                return new kuzzle_1.HttpStream(stream);
            }
            catch (error) {
                // ? Like this endpoint is mostly called by browser prefer return raw message for essyier readable error
                request.response.configure({
                    format: "raw",
                    headers: {
                        "Content-Type": "text/plain",
                    },
                    status: error.status,
                });
                return error.message;
            }
        }
        const id = request.getId();
        const startAt = request.input.args.startAt
            ? request.getDate("startAt")
            : null;
        const endAt = request.input.args.endAt ? request.getDate("endAt") : null;
        const query = request.input.body?.query;
        const sort = request.input.body?.sort;
        const type = request.input.args.type;
        const lang = request.getLangParam();
        const link = await this.measureExporter.prepareExport(engineId, request.getUser(), {
            endAt,
            id,
            lang,
            query,
            sort,
            startAt,
            type,
        });
        return { link };
    }
    async receiveMeasures(request) {
        const engineId = request.getString("engineId");
        const deviceId = request.getId();
        const measures = request.getBodyArray("measures");
        const payloadUuids = request.getBodyArray("payloadUuids", []);
        if (payloadUuids.length === 0) {
            payloadUuids.push(request.id);
        }
        for (let i = 0; i < measures.length; i++) {
            if (typeof measures[i].measureName !== "string") {
                throw new kuzzle_1.BadRequestError(`body.measures[${i}].measureName must be a string`);
            }
            if (typeof measures[i].measuredAt === "undefined") {
                measures[i].measuredAt = Date.now();
            }
            else if (typeof measures[i].measuredAt !== "number") {
                throw new kuzzle_1.BadRequestError(`body.measures[${i}].measuredAt must be a number`);
            }
            if (typeof measures[i].type !== "string") {
                throw new kuzzle_1.BadRequestError(`body.measures[${i}].type must be a string`);
            }
            if (!lodash_1.default.isPlainObject(measures[i].values)) {
                throw new kuzzle_1.BadRequestError(`body.measures[${i}].values must be an object`);
            }
        }
        await this.deviceService.receiveMeasures(engineId, deviceId, measures, payloadUuids, request);
    }
    async export(request) {
        const engineId = request.getString("engineId");
        if (request.context.connection.protocol === "http" &&
            request.context.connection.misc.verb === "GET") {
            try {
                const exportId = request.getString("exportId");
                const stream = await this.exporter.sendExport(engineId, exportId);
                request.response.configure({
                    headers: {
                        "Content-Disposition": `attachment; filename="${plugin_1.InternalCollection.DEVICES}.csv"`,
                        "Content-Type": "text/csv",
                    },
                });
                return new kuzzle_1.HttpStream(stream);
            }
            catch (error) {
                // ? Like this endpoint is mostly called by browser prefer return raw message for essyier readable error
                request.response.configure({
                    format: "raw",
                    headers: {
                        "Content-Type": "text/plain",
                    },
                    status: error.status,
                });
                return error.message;
            }
        }
        const query = request.input.body?.query;
        const sort = request.input.body?.sort;
        const lang = request.getLangParam();
        const link = await this.exporter.prepareExport(engineId, request.getUser(), {
            lang,
            query,
            sort,
        });
        return { link };
    }
    async getLastMeasuredAt(request) {
        const deviceId = request.getId();
        const engineId = request.getString("engineId");
        const lastMeasuredAt = await this.deviceService.getLastMeasuredAt(engineId, deviceId);
        return {
            lastMeasuredAt,
        };
    }
    async mGetLastMeasuredAt(request) {
        const engineId = request.getString("engineId");
        const deviceIds = request.getBodyArray("ids");
        return this.deviceService.mGetLastMeasuredAt(engineId, deviceIds);
    }
}
exports.DevicesController = DevicesController;
//# sourceMappingURL=DevicesController.js.map