"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MeasureExporter = void 0;
const kuzzle_1 = require("kuzzle");
const kuzzle_plugin_commons_1 = require("kuzzle-plugin-commons");
const lodash_1 = __importDefault(require("lodash"));
const kuzzle_2 = require("kuzzle");
const plugin_1 = require("../plugin");
const shared_1 = require("../shared");
class MeasureExporter extends shared_1.AbstractExporter {
    exportRedisKey(engineId, exportId) {
        return `exports:measures:${engineId}:${exportId}`;
    }
    getLink(engineId, exportId, params) {
        return `/_/device-manager/${engineId}/${this.target}/${params.id}/measures/_export/${exportId}`;
    }
    /**
     * Searches for measures and return them in a standard JSON
     */
    async search(engineId, params, options) {
        const searchQuery = this.prepareMeasureSearch(params);
        const result = await this.sdk.document.search(engineId, plugin_1.InternalCollection.MEASURES, {
            query: searchQuery,
            sort: params.sort ?? { measuredAt: "desc" },
        }, {
            from: options?.from ?? 0,
            lang: params.lang,
            size: options?.size ?? 25,
        });
        return { measures: result.hits, total: result.total };
    }
    /**
     * Prepare a measure export by saving the parameters in Redis.
     *
     * We need this method to process the export in 2 steps:
     *   - preparing the export with a POST or WebSocket request
     *   - download the export (only with a browser <a> link)
     *
     * Never return a rejected promise and write potential error on the stream
     */
    async prepareExport(engineId, user, params) {
        const digitalTwin = await this.sdk.document.get(engineId, this.target, params.id);
        const searchQuery = this.prepareMeasureSearch(params);
        const exportParams = {
            id: params.id,
            lang: params.lang,
            model: digitalTwin._source.model,
            query: searchQuery,
            sort: params.sort ?? { measuredAt: "desc" },
        };
        /**
         * ? Prevent error duplicate result on search with sort.
         * "Unable to retrieve all results from search: the sort combination must identify one item only. Add document "_id" to the sort."
         */
        if (exportParams.sort._id === undefined) {
            exportParams.sort._id = "asc";
        }
        return super.prepareExport(engineId, user, exportParams);
    }
    formatHit(columns, hit) {
        return columns.map(({ name, shouldCheckName, path, isIsoDate }) => {
            if (shouldCheckName &&
                this.target === plugin_1.InternalCollection.ASSETS &&
                hit._source.asset?.measureName !== name) {
                return null;
            }
            const formattedValue = lodash_1.default.get(hit, path, null);
            if (formattedValue !== null && isIsoDate) {
                return new Date(formattedValue).toISOString();
            }
            return lodash_1.default.get(hit, path, null);
        });
    }
    /**
     * Retrieve a prepared export and write each document as a CSV in the stream
     *
     * This method never returns a rejected promise, but write potential error in
     * the stream.
     */
    async sendExport(engineId, exportId) {
        const { query, sort, model, lang = "elasticsearch", } = await this.getExport(engineId, exportId);
        const result = await this.sdk.document.search(engineId, plugin_1.InternalCollection.MEASURES, { query, sort }, { lang, size: 200 });
        const targetModel = this.target === plugin_1.InternalCollection.ASSETS ? "asset" : "device";
        const engine = await this.getEngine(engineId);
        const modelDocument = await (0, kuzzle_plugin_commons_1.ask)(`ask:device-manager:model:${targetModel}:get`, {
            engineGroup: engine.group,
            model,
        });
        const measureColumns = await this.generateMeasureColumns(modelDocument[targetModel].measures);
        // sometimes we have multiple measures with same type
        // detect them and add them a flag that will be used later
        const foundDuplicateType = [];
        for (const col of measureColumns) {
            if (!foundDuplicateType.includes(col.path)) {
                const pathFilteredColumns = measureColumns.filter((c) => col.path === c.path);
                if (pathFilteredColumns.length > 1) {
                    foundDuplicateType.push(col.path);
                }
            }
        }
        for (const col of measureColumns) {
            if (foundDuplicateType.includes(col.path)) {
                col.shouldCheckName = true;
            }
        }
        const columns = [
            { header: "Measure Id", path: "_id" },
            { header: "Measured At", path: "_source.measuredAt" },
            {
                header: "Measured At ISO",
                isIsoDate: true,
                path: "_source.measuredAt",
            },
            { header: "Measure Type", path: "_source.type" },
            { header: "Device Id", path: "_source.origin._id" },
            { header: "Device Model", path: "_source.origin.deviceModel" },
            { header: "Asset Id", path: "_source.asset._id" },
            { header: "Asset Model", path: "_source.asset.model" },
            ...measureColumns,
        ];
        const stream = this.getExportStream(result, columns);
        await this.sdk.ms.del(this.exportRedisKey(engineId, exportId));
        return stream;
    }
    async generateMeasureColumns(documentMeasures) {
        /**
         * @example
         * {
         *   temperature: ['temperature.type'],
         *   acceleration: [
         *     'acceleration.properties.x.type',
         *     'acceleration.properties.y.type',
         *     'acceleration.properties.z.type',
         *     'accuracy.type'
         *   ]
         * }
         */
        const mappingsByMeasureType = {};
        const measures = [];
        for (const { name, type } of documentMeasures) {
            if (!(type in mappingsByMeasureType)) {
                const { measure } = await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:model:measure:get", { type });
                mappingsByMeasureType[type] = Object.keys((0, shared_1.flattenObject)(measure.valuesMappings));
            }
            for (const measure of mappingsByMeasureType[type]) {
                /**
                 * The path to be used to retrieve the measure values in a Measure Document
                 *
                 * @example
                 * 'temperature', 'acceleration.x', 'accuracy'
                 */
                const path = measure.replace(".type", "").replace(".properties", "");
                const header = `${name}.${path}`;
                measures.push({
                    header,
                    name,
                    path: `_source.values.${path}`,
                    shouldCheckName: false,
                });
            }
        }
        return measures;
    }
    prepareMeasureSearch(params) {
        if (!this.target) {
            throw new kuzzle_1.InternalError('Missing "target" parameter');
        }
        const measuredAtRange = {
            range: {
                measuredAt: {
                    gte: 0,
                    lte: Number.MAX_SAFE_INTEGER,
                },
            },
        };
        if (params.startAt) {
            measuredAtRange.range.measuredAt.gte = new Date(params.startAt).getTime();
        }
        if (params.endAt) {
            measuredAtRange.range.measuredAt.lte = new Date(params.endAt).getTime();
        }
        const searchQuery = {
            and: [measuredAtRange],
        };
        if (this.target === plugin_1.InternalCollection.ASSETS) {
            searchQuery.and.push({
                equals: { "asset._id": params.id },
            });
        }
        else {
            searchQuery.and.push({
                equals: { "origin._id": params.id },
            });
        }
        if (params.type) {
            searchQuery.and.push({ equals: { type: params.type } });
        }
        // ? Like previous filters are write in koncorde we need to translate to elasticsearch when query is in this language
        if (params.lang === "elasticsearch") {
            const esQuery = new kuzzle_2.QueryTranslator().translate(searchQuery);
            if (params.query !== undefined) {
                esQuery.bool.filter.push(params.query);
            }
            return esQuery;
        }
        if (params.query !== undefined) {
            searchQuery.and.push(params.query);
        }
        return searchQuery;
    }
}
exports.MeasureExporter = MeasureExporter;
//# sourceMappingURL=MeasureExporter.js.map