"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MeasureService = void 0;
const kuzzle_1 = require("kuzzle");
const kuzzle_plugin_commons_1 = require("kuzzle-plugin-commons");
const lodash_1 = __importDefault(require("lodash"));
const asset_1 = require("../asset");
const plugin_1 = require("../plugin");
const shared_1 = require("../shared");
const MeasureSources_1 = require("./types/MeasureSources");
const MeasureSourcesBuilder_1 = require("./MeasureSourcesBuilder");
const MeasureTarget_1 = require("./types/MeasureTarget");
const MeasureTargetBuilder_1 = require("./MeasureTargetBuilder");
class MeasureService extends shared_1.BaseService {
    constructor(plugin) {
        super(plugin);
        (0, kuzzle_plugin_commons_1.onAsk)("device-manager:measures:sourceIngest", async (payload) => {
            if (!payload) {
                return;
            }
            if ((0, MeasureSources_1.isSourceApi)(payload.source) && (0, MeasureTarget_1.isTargetApi)(payload.target)) {
                await this.ingestApi(payload.source, payload.target, payload.measurements, payload.payloadUuids);
            }
        });
        (0, kuzzle_plugin_commons_1.onAsk)("device-manager:measures:ingest", async (payload) => {
            if (!payload) {
                return;
            }
            await this.ingest(payload.device, payload.measurements, payload.metadata, payload.payloadUuids);
        });
    }
    /**
     * Register new measures from an API, updates :
     * - asset
     * - engine measures
     *
     * This method represents the ingestion pipeline:
     *  - trigger events `before` (measure enrichment)
     *  - save documents (measures and asset)
     *  - trigger events `after`
     */
    async ingestApi(source, target, measurements, payloadUuids) {
        const { id: dataSourceId } = source;
        const { indexId, assetId } = target;
        if (!measurements) {
            this.app.log.warn(`No measurements provided for "${dataSourceId}" API measures ingest`);
            return;
        }
        const assetDocument = await this.findAsset(indexId, assetId);
        if (!assetDocument) {
            throw new kuzzle_1.BadRequestError(`Asset "${assetId}" does not exists on index "${indexId}"`);
        }
        const asset = assetDocument._source;
        const measures = await this.buildApiMeasures(source, assetDocument, measurements, payloadUuids, target.engineGroup);
        asset.measures || (asset.measures = {});
        /**
         * Event before starting to process new measures.
         *
         * Useful to enrich measures before they are saved.
         */
        await this.app.trigger("device-manager:measures:process:sourceBefore", { asset, measures, source, target });
        if (indexId) {
            await this.app.trigger(`engine:${indexId}:device-manager:measures:process:sourceBefore`, { asset, measures, source, target });
        }
        const assetStates = this.updateAssetMeasures(assetDocument, measures);
        await this.app.trigger("device-manager:measures:persist:sourceBefore", { asset, measures, source, target });
        if (indexId) {
            await this.app.trigger(`engine:${indexId}:device-manager:measures:persist:sourceBefore`, { asset, measures, source, target });
        }
        const promises = [];
        if (indexId) {
            promises.push(this.sdk.document
                .mCreate(indexId, plugin_1.InternalCollection.MEASURES, measures.map((measure) => ({ body: measure })))
                .then(({ errors }) => {
                if (errors.length !== 0) {
                    throw new kuzzle_1.BadRequestError(`Cannot save measures: ${errors[0].reason}`);
                }
            }));
            promises.push(this.mutexUpdateAsset(indexId, assetId, asset));
            promises.push(this.historizeAssetStates(assetStates, indexId, JSON.parse(JSON.stringify(asset.metadata)), asset.metadata));
        }
        await Promise.all(promises);
        /**
         * Event at the end of the measure process pipeline.
         *
         * Useful to trigger business rules like alerts
         *
         * @todo test this
         */
        await this.app.trigger("device-manager:measures:process:sourceAfter", { asset, measures, source, target });
        if (indexId) {
            await this.app.trigger(`engine:${indexId}:device-manager:measures:process:sourceAfter`, { asset, measures, source, target });
        }
    }
    /**
     * Register new measures from a device, updates :
     * - admin device
     * - engine device
     * - linked asset
     * - engine measures
     *
     * A mutex ensure that a device can ingest one measure at a time.
     *
     * This method represents the ingestion pipeline:
     *  - build measures documents and update digital twins (device and asset)
     *  - trigger events `before` (measure enrichment)
     *  - save documents (measures, device and asset)
     *  - trigger events `after`
     *
     * @deprecated
     */
    async ingest(device, measurements, metadata, payloadUuids) {
        await (0, shared_1.lock)(`measure:ingest:${device._id}`, async () => {
            var _a;
            if (!measurements) {
                this.app.log.warn(`Cannot find measurements for device "${device._source.reference}"`);
                return;
            }
            const { engineId, reference, model, assetId, lastMeasuredAt } = device._source;
            const asset = assetId ? await this.findAsset(engineId, assetId) : null;
            const assetContent = asset?._source;
            const originalAssetMetadata = asset === null
                ? {}
                : JSON.parse(JSON.stringify(asset._source.metadata));
            lodash_1.default.merge(device._source.metadata, metadata);
            const measures = this.buildMeasures(device, asset, measurements, payloadUuids);
            if (asset) {
                (_a = asset._source).measures || (_a.measures = {});
            }
            const source = (0, MeasureSourcesBuilder_1.toDeviceSource)(device._id, reference, model, device._source.metadata, lastMeasuredAt);
            const target = (0, MeasureTargetBuilder_1.toDeviceTarget)(device._source.engineId, device._source.assetId ?? undefined);
            /**
             * Event before starting to process new measures.
             *
             * Useful to enrich measures before they are saved.
             */
            await this.app.trigger("device-manager:measures:process:before", { asset, device, measures });
            if (engineId) {
                await this.app.trigger(`engine:${engineId}:device-manager:measures:process:before`, { asset, device, measures });
            }
            /**
             * Here are the new process before triggers using sources
             *
             * Event before starting to process new measures.
             *
             * Useful to enrich measures before they are saved.
             */
            await this.app.trigger("device-manager:measures:process:sourceBefore", { asset: assetContent, measures, source, target });
            if (engineId) {
                await this.app.trigger(`engine:${engineId}:device-manager:measures:process:sourceBefore`, { asset: assetContent, measures, source, target });
            }
            await this.updateDeviceMeasures(device, measures);
            let assetStates = new Map();
            if (asset) {
                assetStates = await this.updateAssetMeasures(asset, measures);
            }
            await this.app.trigger("device-manager:measures:persist:before", {
                asset,
                device,
                measures,
            });
            if (engineId) {
                await this.app.trigger(`engine:${engineId}:device-manager:measures:persist:before`, { asset, device, measures });
            }
            /**
             * Here are the new persist before triggers using sources
             */
            await this.app.trigger("device-manager:measures:persist:sourceBefore", { asset: assetContent, measures, source, target });
            if (engineId) {
                await this.app.trigger(`engine:${engineId}:device-manager:measures:persist:sourceBefore`, { asset: assetContent, measures, source, target });
            }
            const promises = [];
            promises.push(this.sdk.document
                .update(this.config.adminIndex, plugin_1.InternalCollection.DEVICES, device._id, device._source)
                .catch((error) => {
                throw (0, shared_1.keepStack)(error, new kuzzle_1.BadRequestError(`Cannot update device "${device._id}": ${error.message}`));
            }));
            if (engineId) {
                promises.push(this.sdk.document
                    .update(engineId, plugin_1.InternalCollection.DEVICES, device._id, device._source)
                    .catch((error) => {
                    throw (0, shared_1.keepStack)(error, new kuzzle_1.BadRequestError(`Cannot update engine device "${device._id}": ${error.message}`));
                }));
                promises.push(this.sdk.document
                    .mCreate(engineId, plugin_1.InternalCollection.MEASURES, measures.map((measure) => ({ body: measure })))
                    .then(({ errors }) => {
                    if (errors.length !== 0) {
                        throw new kuzzle_1.BadRequestError(`Cannot save measures: ${errors[0].reason}`);
                    }
                }));
                if (asset) {
                    promises.push(this.mutexUpdateAsset(engineId, asset._id, asset._source));
                    promises.push(this.historizeAssetStates(assetStates, engineId, originalAssetMetadata, asset._source.metadata));
                }
            }
            await Promise.all(promises);
            /**
             * Event at the end of the measure process pipeline.
             *
             * Useful to trigger business rules like alerts
             *
             * @todo test this
             */
            await this.app.trigger("device-manager:measures:process:after", {
                asset,
                device,
                measures,
            });
            if (engineId) {
                await this.app.trigger(`engine:${engineId}:device-manager:measures:process:after`, { asset, device, measures });
            }
            /**
             * Here are the new process after triggers using sources
             *
             * Event at the end of the measure process pipeline.
             *
             * Useful to trigger business rules like alerts
             *
             * @todo test this
             */
            await this.app.trigger("device-manager:measures:process:sourceAfter", { asset: assetContent, measures, source, target });
            if (engineId) {
                await this.app.trigger(`engine:${engineId}:device-manager:measures:process:sourceAfter`, { asset: assetContent, measures, source, target });
            }
        });
    }
    updateDeviceMeasures(device, measurements) {
        if (!device._source.measures) {
            device._source.measures = {};
        }
        let lastMeasuredAt = device._source.lastMeasuredAt ?? 0;
        for (const measurement of measurements) {
            if (measurement.origin.type !== "device") {
                continue;
            }
            const measureName = measurement.origin.measureName;
            const previousMeasure = device._source.measures[measureName];
            if (previousMeasure &&
                previousMeasure.measuredAt >= measurement.measuredAt) {
                continue;
            }
            if (measurement.measuredAt > lastMeasuredAt) {
                lastMeasuredAt = measurement.measuredAt;
            }
            device._source.measures[measureName] = {
                measuredAt: measurement.measuredAt,
                name: measureName,
                originId: measurement.origin._id,
                payloadUuids: measurement.origin.payloadUuids,
                type: measurement.type,
                values: measurement.values,
            };
        }
        device._source.lastMeasuredAt = lastMeasuredAt;
    }
    // @todo there shouldn't be any logic related to asset historization here, but no other choices for now. It needs to be re-architected
    /**
     * Update asset with each non-null non-computed measures.
     *
     * @returns A map of each asset state by measuredAt timestamp.
     */
    updateAssetMeasures(asset, measurements) {
        // We use a Map in order to preserve the insertion order to avoid another sort
        const assetStates = new Map();
        if (!asset._source.measures) {
            asset._source.measures = {};
        }
        let lastMeasuredAt = asset._source.lastMeasuredAt ?? 0;
        for (const measurement of measurements) {
            if (measurement.origin.type === "computed") {
                continue;
            }
            if (measurement.asset === null) {
                continue;
            }
            const measureName = measurement.asset?.measureName ?? null;
            // The measurement was not present in the asset device links so it should
            // not be saved in the asset measures
            if (measureName === null) {
                continue;
            }
            const previousMeasure = asset._source.measures[measureName];
            if (previousMeasure &&
                previousMeasure.measuredAt >= measurement.measuredAt) {
                continue;
            }
            if (measurement.measuredAt > lastMeasuredAt) {
                lastMeasuredAt = measurement.measuredAt;
            }
            asset._source.measures[measureName] = {
                measuredAt: measurement.measuredAt,
                name: measureName,
                originId: measurement.origin._id,
                payloadUuids: measurement.origin.payloadUuids,
                type: measurement.type,
                values: measurement.values,
            };
            assetStates.set(measurement.measuredAt, JSON.parse(JSON.stringify(asset)));
        }
        asset._source.lastMeasuredAt = lastMeasuredAt;
        return assetStates;
    }
    /**
     * Build the measures document received from the API
     *
     * @param source The API data source
     * @param asset The target asset to build the measure for
     * @param measure The decoded raw measures
     * @param payloadUuids The uuid's of the payloads used to create the measure
     *
     * @returns A MeasurementContent builded from parameters
     */
    async buildApiMeasures(source, asset, measures, payloadUuids, engineGroup) {
        const apiMeasures = [];
        for (const measure of measures) {
            let assetContext = null;
            const isInModel = await this.isMeasureNameInModel(measure.measureName, asset._source.model, engineGroup);
            if (isInModel) {
                assetContext = asset_1.AssetSerializer.measureContext(asset, measure.measureName);
            }
            const measureSource = (0, MeasureSourcesBuilder_1.apiSourceToOriginApi)(source, payloadUuids);
            apiMeasures.push({
                asset: assetContext,
                measuredAt: measure.measuredAt,
                origin: measureSource,
                type: measure.type,
                values: measure.values,
            });
        }
        return apiMeasures;
    }
    /**
     * Build the measures documents to save
     */
    buildMeasures(device, asset, measurements, payloadUuids) {
        const measures = [];
        for (const measurement of measurements) {
            // @todo check if measure type exists
            let assetContext = null;
            if (asset) {
                const assetMeasureName = this.findAssetMeasureNameFromDevice(device._id, measurement.measureName, asset._source);
                if (assetMeasureName) {
                    assetContext = asset_1.AssetSerializer.measureContext(asset, assetMeasureName);
                }
            }
            const measureContent = {
                asset: assetContext,
                measuredAt: measurement.measuredAt,
                origin: {
                    _id: device._id,
                    deviceMetadata: device._source.metadata,
                    deviceModel: device._source.model,
                    measureName: measurement.measureName,
                    payloadUuids,
                    reference: device._source.reference,
                    type: "device",
                },
                type: measurement.type,
                values: measurement.values,
            };
            measures.push(measureContent);
        }
        return measures.sort((measureA, measureB) => measureA.measuredAt - measureB.measuredAt);
    }
    /**
     * Find an asset by its ID and its engine ID.
     *
     * @param engineId The target index ID
     * @param assetId the target asset ID
     * @returns The asset or null if not found
     */
    async findAsset(engineId, assetId) {
        try {
            const asset = await this.sdk.document.get(engineId, plugin_1.InternalCollection.ASSETS, assetId);
            return asset;
        }
        catch (error) {
            this.app.log.error(`[${engineId}] Cannot find asset "${assetId}".`);
            return null;
        }
    }
    /**
     * Check if the asset measure name is associated to the asset model
     *
     * @param measureName The measure name to check
     * @param model The asset model the measureName should belong
     *
     * @returns True if the asset measure name belongs to the asset model, false otherwise
     * @throws If the model does not exists
     */
    async isMeasureNameInModel(measureName, model, engineGroup = "commons") {
        const assetModel = await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:model:asset:get", {
            engineGroup,
            model: model,
        });
        if (!assetModel) {
            throw new kuzzle_1.BadRequestError(`Model "${model}" does not exists`);
        }
        const assetMeasureName = assetModel.asset.measures.find((m) => m.name === measureName);
        return assetMeasureName?.name ? true : false;
    }
    /**
     * Find the asset measure name from the device and its measure type
     *
     * @param deviceId The source device ID
     * @param measureType The device measure type
     * @param asset The asset the device is linked to
     *
     * @returns The asset measure name or null if it does not belong to the link
     * @throws If the device is not linked to the asset
     */
    findAssetMeasureNameFromDevice(deviceId, measureType, asset) {
        const linkedDevice = asset.linkedDevices.find((link) => link._id === deviceId);
        if (!linkedDevice) {
            throw new kuzzle_1.BadRequestError(`Device "${deviceId}" is not linked to "${asset.model}" asset: "${asset.reference}"`);
        }
        const assetMeasureName = linkedDevice.measureNames.find((m) => m.device === measureType);
        return assetMeasureName?.asset ?? null;
    }
    mutexUpdateAsset(engineId, assetId, asset) {
        // ? Potential race condition if 2 differents device are linked
        // ? to the same asset and get processed at the same time.
        //
        // ? Use mutex to try to fix update conflict
        return (0, shared_1.lock)(`asset:${engineId}:${assetId}`, async () => this.sdk.document
            .update(engineId, plugin_1.InternalCollection.ASSETS, assetId, asset)
            .catch((error) => {
            throw (0, shared_1.keepStack)(error, new kuzzle_1.BadRequestError(`Cannot update asset "${assetId}": ${error.message}`));
        }));
    }
    /**
     * Create a new document in the collection asset-history, for each asset state
     */
    async historizeAssetStates(assetStates, engineId, originalAssetMetadata, assetMetadata) {
        const metadataChanges = (0, shared_1.objectDiff)(originalAssetMetadata, assetMetadata);
        if (!this.config.assetsHistorizesMeasures) {
            // If there are no metadata changes, nothing to do.
            if (metadataChanges.length === 0) {
                return;
            }
            const timestamps = Array.from(assetStates.keys());
            const lastTimestampRecorded = timestamps[timestamps.length - 1];
            const lastAssetState = assetStates.get(lastTimestampRecorded);
            if (!lastAssetState) {
                return;
            }
            // Update the asset's metadata to the new one.
            lastAssetState._source.metadata = assetMetadata;
            // Create a metadata event.
            const event = {
                metadata: {
                    names: metadataChanges,
                },
                name: "metadata",
            };
            const history = {
                asset: lastAssetState._source,
                event,
                id: lastAssetState._id,
                timestamp: lastTimestampRecorded,
            };
            return (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:asset:history:add", {
                engineId,
                histories: [history],
            });
        }
        // Otherwise, we record measure events (and metadata on the last state if needed)
        const timestamps = Array.from(assetStates.keys());
        const lastTimestampRecorded = timestamps[timestamps.length - 1];
        const histories = [];
        for (const [measuredAt, assetState] of assetStates) {
            // Gather measure names corresponding to the current timestamp.
            const measureNames = [];
            for (const measure of Object.values(assetState._source.measures)) {
                if (measure?.measuredAt === measuredAt) {
                    measureNames.push(measure.name);
                }
            }
            // Build the measure event.
            const event = {
                measure: {
                    names: measureNames,
                },
                name: "measure",
            };
            // Initialize with the original metadata.
            assetState._source.metadata = originalAssetMetadata;
            // If we are at the last recorded state and there are metadata changes,
            // update both the event and asset metadata.
            if (metadataChanges.length !== 0 &&
                measuredAt === lastTimestampRecorded) {
                event.metadata = {
                    names: metadataChanges,
                };
                assetState._source.metadata = assetMetadata;
            }
            histories.push({
                asset: assetState._source,
                event,
                id: assetState._id,
                timestamp: measuredAt,
            });
        }
        return (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:asset:history:add", {
            engineId,
            // Reverse order because measuredAt is sorted in ascending order,
            // while in mCreate the last item will be the first document to be created
            histories: histories.reverse(),
        });
    }
}
exports.MeasureService = MeasureService;
//# sourceMappingURL=MeasureService.js.map