"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelService = void 0;
const kuzzle_1 = require("kuzzle");
const kuzzle_plugin_commons_1 = require("kuzzle-plugin-commons");
const plugin_1 = require("../plugin");
const shared_1 = require("../shared");
const ModelSerializer_1 = require("./ModelSerializer");
const MappingsConflictsError_1 = require("./MappingsConflictsError");
const AJValidator_1 = require("../shared/utils/AJValidator");
const SchemaValidationError_1 = require("../shared/errors/SchemaValidationError");
const MeasuresDuplicates_1 = require("./MeasuresDuplicates");
const MeasuresNamesDuplicatesError_1 = require("./MeasuresNamesDuplicatesError");
class ModelService extends shared_1.BaseService {
    constructor(plugin) {
        super(plugin);
        this.registerAskEvents();
    }
    registerAskEvents() {
        (0, kuzzle_plugin_commons_1.onAsk)("ask:device-manager:model:asset:get", async ({ engineGroup, model }) => {
            const assetModel = await this.getAsset(engineGroup, model);
            return assetModel._source;
        });
        (0, kuzzle_plugin_commons_1.onAsk)("ask:device-manager:model:device:get", async ({ model }) => {
            const deviceModel = await this.getDevice(model);
            return deviceModel._source;
        });
        (0, kuzzle_plugin_commons_1.onAsk)("ask:device-manager:model:group:get", async ({ model }) => {
            const groupModel = await this.getGroup(model);
            return groupModel._source;
        });
        (0, kuzzle_plugin_commons_1.onAsk)("ask:device-manager:model:measure:get", async ({ type }) => {
            const measureModel = await this.getMeasure(type);
            return measureModel._source;
        });
        const genericModelsHandler = async (documents, request) => {
            const { index, collection } = request.input.args;
            if (index === this.config.adminIndex && collection === "models") {
                const models = documents.map((elt) => {
                    return elt._source;
                });
                await this.checkModelsConflicts(models);
            }
            return documents;
        };
        this.app.pipe.register("generic:document:beforeWrite", genericModelsHandler);
        this.app.pipe.register("generic:document:beforeUpdate", genericModelsHandler);
        this.app.hook.register("generic:document:afterUpdate", async (documents, request) => {
            const { index, collection } = request.input.args;
            if (index === this.config.adminIndex && collection === "models") {
                await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:engine:updateAll");
            }
        });
    }
    async checkModelsConflicts(documents) {
        const assets = documents.filter((elt) => {
            return elt.type === "asset";
        });
        const devices = documents.filter((elt) => {
            return elt.type === "device";
        });
        const groups = documents.filter((elt) => {
            return elt.type === "group";
        });
        const measures = documents.filter((elt) => {
            return elt.type === "measure";
        });
        if (assets.length > 0) {
            const conflicts = await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:engine:doesUpdateConflict", {
                twin: {
                    models: assets,
                    type: "asset",
                },
            });
            if (conflicts.length > 0) {
                throw new MappingsConflictsError_1.MappingsConflictsError(`New assets mappings are causing conflicts`, conflicts);
            }
        }
        if (devices.length > 0) {
            const conflicts = await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:engine:doesUpdateConflict", {
                twin: {
                    models: devices,
                    type: "device",
                },
            });
            if (conflicts.length > 0) {
                throw new MappingsConflictsError_1.MappingsConflictsError(`New devices mappings are causing conflicts`, conflicts);
            }
        }
        if (groups.length > 0) {
            const conflicts = await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:engine:doesUpdateConflict", {
                groupModels: groups,
            });
            if (conflicts.length > 0) {
                throw new MappingsConflictsError_1.MappingsConflictsError(`New group mappings are causing conflicts`, conflicts);
            }
        }
        if (measures.length > 0) {
            const conflicts = await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:engine:doesUpdateConflict", {
                measuresModels: measures,
            });
            if (conflicts.length > 0) {
                throw new MappingsConflictsError_1.MappingsConflictsError(`New measures mappings are causing conflicts`, conflicts);
            }
        }
    }
    async writeAsset(engineGroup, model, metadataMappings, defaultMetadata, metadataDetails, metadataGroups, measures, tooltipModels, locales) {
        if (kuzzle_1.Inflector.pascalCase(model) !== model) {
            throw new kuzzle_1.BadRequestError(`Asset model "${model}" must be PascalCase.`);
        }
        const duplicates = (0, MeasuresDuplicates_1.getNamedMeasuresDuplicates)(measures);
        if (duplicates.length > 0) {
            throw new MeasuresNamesDuplicatesError_1.MeasuresNamesDuplicatesError("Asset model measures contain one or multiple duplicate measure name", duplicates);
        }
        const modelContent = {
            asset: {
                defaultMetadata,
                locales,
                measures,
                metadataDetails,
                metadataGroups,
                metadataMappings,
                model,
                tooltipModels,
            },
            engineGroup,
            type: "asset",
        };
        this.checkDefaultValues(metadataMappings, defaultMetadata);
        const conflicts = await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:engine:doesUpdateConflict", { twin: { models: [modelContent], type: "asset" } });
        if (conflicts.length > 0) {
            throw new MappingsConflictsError_1.MappingsConflictsError(`New assets mappings are causing conflicts`, conflicts);
        }
        const assetModel = await this.sdk.document.createOrReplace(this.config.adminIndex, plugin_1.InternalCollection.MODELS, ModelSerializer_1.ModelSerializer.id("asset", modelContent), modelContent);
        await this.sdk.collection.refresh(this.config.adminIndex, plugin_1.InternalCollection.MODELS);
        await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:engine:updateAll");
        await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:asset:refresh-model", {
            assetModel: modelContent,
        });
        return assetModel;
    }
    checkDefaultValues(metadataMappings, defaultMetadata) {
        const flattenedMetadataMappings = (0, shared_1.flattenObject)(metadataMappings);
        const metadata = Object.keys(JSON.parse(JSON.stringify(flattenedMetadataMappings)
            .replace(/properties\./g, "")
            .replace(/\.type/g, "")));
        const values = Object.keys((0, shared_1.flattenObject)(defaultMetadata));
        for (let i = 0; i < values.length; i++) {
            const key = values[i];
            // ? Check if the exact key exists in the metadata
            if (!metadata.includes(key)) {
                // ? Extract base key for complex types like geo_point or geo_shape
                const baseKey = key.includes(".") ? key.split(".")[0] : key;
                // ? Check if the base key is in the metadata
                if (!metadata.includes(baseKey)) {
                    throw new kuzzle_1.BadRequestError(`The default value "${key}" is not in the metadata mappings.`);
                }
                // ? Accept nested properties for geo_point or geo_shape
                const baseKeyMetadata = flattenedMetadataMappings[`${baseKey}.type`];
                if (baseKeyMetadata === "geo_point" ||
                    baseKeyMetadata === "geo_shape") {
                    continue;
                }
                throw new kuzzle_1.BadRequestError(`The default value "${values[i]}" is not in the metadata mappings.`);
            }
        }
    }
    async writeDevice(model, metadataMappings, defaultMetadata, metadataDetails, metadataGroups, measures) {
        if (kuzzle_1.Inflector.pascalCase(model) !== model) {
            throw new kuzzle_1.BadRequestError(`Device model "${model}" must be PascalCase.`);
        }
        const duplicates = (0, MeasuresDuplicates_1.getNamedMeasuresDuplicates)(measures);
        if (duplicates.length > 0) {
            throw new MeasuresNamesDuplicatesError_1.MeasuresNamesDuplicatesError("Device model measures contain one or multiple duplicate measure name", duplicates);
        }
        const modelContent = {
            device: {
                defaultMetadata,
                measures,
                metadataDetails,
                metadataGroups,
                metadataMappings,
                model,
            },
            type: "device",
        };
        const conflicts = await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:engine:doesUpdateConflict", { twin: { models: [modelContent], type: "device" } });
        if (conflicts.length > 0) {
            throw new MappingsConflictsError_1.MappingsConflictsError(`New devices mappings are causing conflicts`, conflicts);
        }
        const deviceModel = await this.sdk.document.createOrReplace(this.config.adminIndex, plugin_1.InternalCollection.MODELS, ModelSerializer_1.ModelSerializer.id("device", modelContent), modelContent);
        await this.sdk.collection.refresh(this.config.adminIndex, plugin_1.InternalCollection.MODELS);
        await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:engine:updateAll");
        return deviceModel;
    }
    async writeGroup(engineGroup, model, metadataMappings, defaultMetadata, metadataDetails, metadataGroups) {
        if (kuzzle_1.Inflector.pascalCase(model) !== model) {
            throw new kuzzle_1.BadRequestError(`Group model "${model}" must be PascalCase.`);
        }
        const modelContent = {
            engineGroup,
            group: {
                defaultMetadata,
                metadataDetails,
                metadataGroups,
                metadataMappings,
                model,
            },
            type: "group",
        };
        const conflicts = await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:engine:doesUpdateConflict", { groupModels: [modelContent] });
        if (conflicts.length > 0) {
            throw new MappingsConflictsError_1.MappingsConflictsError(`New group mappings are causing conflicts`, conflicts);
        }
        const groupModel = await this.sdk.document.createOrReplace(this.config.adminIndex, plugin_1.InternalCollection.MODELS, ModelSerializer_1.ModelSerializer.id("group", modelContent), modelContent);
        await this.sdk.collection.refresh(this.config.adminIndex, plugin_1.InternalCollection.MODELS);
        await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:engine:updateAll");
        return groupModel;
    }
    async writeMeasure(type, valuesMappings, validationSchema, valuesDetails, locales) {
        const modelContent = {
            measure: {
                locales,
                type,
                valuesDetails,
                valuesMappings,
            },
            type: "measure",
        };
        if (validationSchema) {
            try {
                (0, AJValidator_1.addSchemaToCache)(type, validationSchema);
                modelContent.measure.validationSchema = validationSchema;
            }
            catch (error) {
                throw new SchemaValidationError_1.SchemaValidationError("Provided schema is not valid", (0, AJValidator_1.getAJVErrors)());
            }
        }
        const conflicts = await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:engine:doesUpdateConflict", { measuresModels: [modelContent] });
        if (conflicts.length > 0) {
            throw new MappingsConflictsError_1.MappingsConflictsError(`New assets mappings are causing conflicts`, conflicts);
        }
        const measureModel = await this.sdk.document.createOrReplace(this.config.adminIndex, plugin_1.InternalCollection.MODELS, ModelSerializer_1.ModelSerializer.id("measure", modelContent), modelContent);
        await this.sdk.collection.refresh(this.config.adminIndex, plugin_1.InternalCollection.MODELS);
        await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:engine:updateAll");
        return measureModel;
    }
    async deleteAsset(_id) {
        await this.sdk.document.delete(this.config.adminIndex, plugin_1.InternalCollection.MODELS, _id);
    }
    async deleteDevice(_id) {
        await this.sdk.document.delete(this.config.adminIndex, plugin_1.InternalCollection.MODELS, _id);
    }
    async deleteGroup(_id) {
        await this.sdk.document.delete(this.config.adminIndex, plugin_1.InternalCollection.MODELS, _id);
    }
    async deleteMeasure(_id) {
        await this.sdk.document.delete(this.config.adminIndex, plugin_1.InternalCollection.MODELS, _id);
    }
    async listAsset(engineGroup) {
        const result = await this.searchAssets(engineGroup, {
            searchBody: {
                sort: { "asset.model": "asc" },
            },
            size: 5000,
        });
        return result.hits;
    }
    async listDevices() {
        const result = await this.searchDevices({
            searchBody: {
                sort: { "device.model": "asc" },
            },
            size: 5000,
        });
        return result.hits;
    }
    async listGroups(engineGroup) {
        const result = await this.searchGroups(engineGroup, {
            searchBody: {
                sort: { "group.model": "asc" },
            },
            size: 5000,
        });
        return result.hits;
    }
    async listMeasures() {
        const result = await this.searchMeasures({
            searchBody: {
                sort: { "measure.type": "asc" },
            },
            size: 5000,
        });
        return result.hits;
    }
    async searchAssets(engineGroup, searchParams) {
        const query = {
            bool: {
                must: [
                    searchParams.searchBody.query,
                    { match: { type: "asset" } },
                    {
                        bool: {
                            should: [
                                { match: { engineGroup } },
                                { match: { engineGroup: "commons" } },
                            ],
                        },
                    },
                ],
            },
        };
        return this.sdk.document.search(this.config.adminIndex, plugin_1.InternalCollection.MODELS, {
            ...searchParams.searchBody,
            query,
        }, {
            from: searchParams.from,
            lang: "elasticsearch",
            scroll: searchParams.scrollTTL,
            size: searchParams.size,
        });
    }
    async searchDevices(searchParams) {
        const query = {
            bool: {
                must: [searchParams.searchBody.query, { match: { type: "device" } }],
            },
        };
        return this.sdk.document.search(this.config.adminIndex, plugin_1.InternalCollection.MODELS, {
            ...searchParams.searchBody,
            query,
        }, {
            from: searchParams.from,
            lang: "elasticsearch",
            scroll: searchParams.scrollTTL,
            size: searchParams.size,
        });
    }
    async searchGroups(engineGroup, searchParams) {
        const query = {
            bool: {
                must: [
                    searchParams.searchBody.query,
                    { match: { type: "group" } },
                    {
                        bool: {
                            should: [
                                { match: { engineGroup } },
                                { match: { engineGroup: "commons" } },
                            ],
                        },
                    },
                ],
            },
        };
        return this.sdk.document.search(this.config.adminIndex, plugin_1.InternalCollection.MODELS, {
            ...searchParams.searchBody,
            query,
        }, {
            from: searchParams.from,
            lang: "elasticsearch",
            scroll: searchParams.scrollTTL,
            size: searchParams.size,
        });
    }
    async searchMeasures(searchParams) {
        const query = {
            bool: {
                must: [searchParams.searchBody.query, { match: { type: "measure" } }],
            },
        };
        return this.sdk.document.search(this.config.adminIndex, plugin_1.InternalCollection.MODELS, {
            ...searchParams.searchBody,
            query,
        }, {
            from: searchParams.from,
            lang: "elasticsearch",
            scroll: searchParams.scrollTTL,
            size: searchParams.size,
        });
    }
    async assetExists(model) {
        const query = {
            and: [
                { equals: { type: "asset" } },
                { equals: { "asset.model": model } },
            ],
        };
        const result = await this.sdk.document.search(this.config.adminIndex, plugin_1.InternalCollection.MODELS, { query }, { lang: "koncorde", size: 1 });
        return result.total > 0;
    }
    async deviceExists(model) {
        const query = {
            and: [
                { equals: { type: "device" } },
                { equals: { "device.model": model } },
            ],
        };
        const result = await this.sdk.document.search(this.config.adminIndex, plugin_1.InternalCollection.MODELS, { query }, { lang: "koncorde", size: 1 });
        return result.total > 0;
    }
    async getAsset(engineGroup, model) {
        const query = {
            and: [
                {
                    or: [
                        { equals: { engineGroup } },
                        { equals: { engineGroup: "commons" } },
                    ],
                },
                { equals: { type: "asset" } },
                { equals: { "asset.model": model } },
            ],
        };
        const result = await this.sdk.document.search(this.config.adminIndex, plugin_1.InternalCollection.MODELS, { query }, { lang: "koncorde", size: 1 });
        if (result.total === 0) {
            throw new kuzzle_1.NotFoundError(`Unknown Asset model "${model}".`);
        }
        return result.hits[0];
    }
    async getDevice(model) {
        const query = {
            and: [
                { equals: { type: "device" } },
                { equals: { "device.model": model } },
            ],
        };
        const result = await this.sdk.document.search(this.config.adminIndex, plugin_1.InternalCollection.MODELS, { query }, { lang: "koncorde", size: 1 });
        if (result.total === 0) {
            throw new kuzzle_1.NotFoundError(`Unknown Device model "${model}".`);
        }
        return result.hits[0];
    }
    async getGroup(model) {
        const query = {
            and: [
                { equals: { type: "group" } },
                { equals: { "group.model": model } },
            ],
        };
        const result = await this.sdk.document.search(this.config.adminIndex, plugin_1.InternalCollection.MODELS, { query }, { lang: "koncorde", size: 1 });
        if (result.total === 0) {
            throw new kuzzle_1.NotFoundError(`Unknown Group model "${model}".`);
        }
        if (result.total > 1) {
            app.log.warn("More than 1 group definition have been found for this model");
        }
        return result.hits[0];
    }
    async getMeasure(type) {
        const query = {
            and: [
                { equals: { type: "measure" } },
                { equals: { "measure.type": type } },
            ],
        };
        const result = await this.sdk.document.search(this.config.adminIndex, plugin_1.InternalCollection.MODELS, { query }, { lang: "koncorde", size: 1 });
        if (result.total === 0) {
            throw new kuzzle_1.NotFoundError(`Unknown Measure type "${type}".`);
        }
        return result.hits[0];
    }
    /**
     * Update an asset model
     */
    async updateAsset(engineGroup, model, metadataMappings, defaultMetadata, metadataDetails, metadataGroups, measures, tooltipModels, locales, request) {
        if (kuzzle_1.Inflector.pascalCase(model) !== model) {
            throw new kuzzle_1.BadRequestError(`Asset model "${model}" must be PascalCase.`);
        }
        this.checkDefaultValues(metadataMappings, defaultMetadata);
        const existingAsset = await this.getAsset(engineGroup, model);
        // The field must be deleted if an element of the table is to be deleted
        await this.sdk.document.deleteFields(this.config.adminIndex, plugin_1.InternalCollection.MODELS, existingAsset._id, ["asset.tooltipModels"], { source: true });
        const measuresUpdated = measures.length === 0 ? existingAsset._source.asset.measures : measures;
        const assetModelContent = {
            asset: {
                defaultMetadata,
                locales,
                measures: measuresUpdated,
                metadataDetails,
                metadataGroups,
                metadataMappings,
                model,
                tooltipModels,
            },
            engineGroup,
            type: "asset",
        };
        const assetModel = {
            _id: existingAsset._id,
            _source: assetModelContent,
        };
        const conflicts = await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:engine:doesUpdateConflict", { twin: { models: [assetModelContent], type: "asset" } });
        if (conflicts.length > 0) {
            throw new MappingsConflictsError_1.MappingsConflictsError(`Assets mappings are causing conflicts`, conflicts);
        }
        const endDocument = await this.updateDocument(request, assetModel, {
            collection: plugin_1.InternalCollection.MODELS,
            engineId: this.config.adminIndex,
        }, { source: true });
        await this.sdk.collection.refresh(this.config.adminIndex, plugin_1.InternalCollection.MODELS);
        await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:engine:updateAll");
        await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:asset:refresh-model", {
            assetModel: assetModelContent,
        });
        return endDocument;
    }
}
exports.ModelService = ModelService;
//# sourceMappingURL=ModelService.js.map