"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getMeasureConflicts = exports.getGroupConflicts = exports.getTwinConflicts = void 0;
const lodash_1 = __importDefault(require("lodash"));
/**
 * Return every conflicts between the mappings of models and a new model
 *
 * @param modelType The target model type
 * @param models The already present models used as comparison source
 * @param additional  The new model
 * @param mappingsPath The object path to the mappings
 * @param modelIdPath The object path to the model ID
 * @returns An array of ConflictChunk between current models and the new model
 */
function getModelConflicts(modelType, models, additional, mappingsPath, modelIdPath) {
    const conflicts = [];
    for (const model of models) {
        const baseMappings = lodash_1.default.get(model, mappingsPath);
        const updatedMappings = JSON.parse(JSON.stringify(baseMappings));
        lodash_1.default.merge(updatedMappings, lodash_1.default.get(additional, mappingsPath));
        const mappingsConflict = findConflicts(baseMappings, updatedMappings, mappingsPath);
        if (mappingsConflict.length > 0) {
            conflicts.push({
                conflicts: mappingsConflict,
                modelType: modelType,
                newModel: lodash_1.default.get(additional, modelIdPath),
                sourceModel: lodash_1.default.get(model, modelIdPath),
            });
        }
    }
    return conflicts;
}
/**
 * Return every conflicts between the mappings of twins and a new twin model
 *
 * @param twinType The target twin type
 * @param models The already present twin models
 * @param additional The new asset
 * @returns An array of ConflictChunk between current twins and the new twin
 */
function getTwinConflicts(twinType, models, additional) {
    const metadataPath = twinType + ".metadataMappings";
    const modelIdPath = twinType + ".model";
    return getModelConflicts(twinType, models, additional, metadataPath, modelIdPath);
}
exports.getTwinConflicts = getTwinConflicts;
/**
 * Return every conflicts between the mappings of group models and a new group model
 *
 * @param groups The already present group models
 * @param additional The new group model
 * @returns An array of ConflictChunk between current groups and the new group
 */
function getGroupConflicts(measures, additional) {
    const modelType = "group";
    const mappingsPath = "group.metadataMappings";
    const modelIdPath = "group.model";
    return getModelConflicts(modelType, measures, additional, mappingsPath, modelIdPath);
}
exports.getGroupConflicts = getGroupConflicts;
/**
 * Return every conflicts between the mappings of measure models and a new measure model
 *
 * @param measures The already present measure models
 * @param additional The new measure
 * @returns An array of ConflictChunk between current measures and the new measure
 */
function getMeasureConflicts(measures, additional) {
    const modelType = "measure";
    const mappingsPath = "measure.valuesMappings";
    const modelIdPath = "measure.type";
    return getModelConflicts(modelType, measures, additional, mappingsPath, modelIdPath);
}
exports.getMeasureConflicts = getMeasureConflicts;
/**
 * Find every conflicts between two objects.
 * WARNING: "undefined" type of conflicts are not registered
 * @param sourceObject The source object used to compare
 * @param objectToCompare The target object to compare to
 * @param basePath Optional path to be added before conflicts paths
 * @returns An array of MappingsConflict
 */
function findConflicts(sourceObject, objectToCompare, basePath = "") {
    if (sourceObject === objectToCompare) {
        return [];
    }
    if (lodash_1.default.isEqual(sourceObject, objectToCompare)) {
        return [];
    }
    const conflicts = [];
    function compareObjects(source, toCompare, path = "") {
        //Loop over the source object properties of distinct depth level
        lodash_1.default.forOwn(source, (value, key) => {
            //Construct full path from the previous path and the current key
            const fullPath = path ? `${path}.${key}` : key;
            //Check the object to compare key availibility, if there's nothing return to continue on the next property
            if (!lodash_1.default.has(toCompare, key)) {
                return;
            }
            //If the property to compare is an object, recursively call the method to advance on the next depth level
            if (lodash_1.default.isObject(value) && lodash_1.default.isObject(toCompare[key])) {
                compareObjects(value, toCompare[key], fullPath);
                return;
            }
            //If the property to compare is not an object and is not equal to the source value, add the current/new value with it's corresponding full path
            if (!lodash_1.default.isEqual(value, toCompare[key])) {
                conflicts.push({
                    currentType: source[key],
                    newType: toCompare[key],
                    path: fullPath,
                });
            }
        });
    }
    compareObjects(sourceObject, objectToCompare, basePath);
    return conflicts;
}
//# sourceMappingURL=ModelsConflicts.js.map