"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelsRegister = void 0;
const kuzzle_1 = require("kuzzle");
const plugin_1 = require("../plugin");
const ModelSerializer_1 = require("./ModelSerializer");
const AJValidator_1 = require("../shared/utils/AJValidator");
const SchemaValidationError_1 = require("../shared/errors/SchemaValidationError");
const MeasuresDuplicates_1 = require("./MeasuresDuplicates");
const MeasuresNamesDuplicatesError_1 = require("./MeasuresNamesDuplicatesError");
class ModelsRegister {
    constructor() {
        this.assetModels = [];
        this.deviceModels = [];
        this.groupModels = [];
        this.measureModels = [];
    }
    get sdk() {
        return this.context.accessors.sdk;
    }
    init(plugin) {
        this.config = plugin.config;
        this.context = plugin.context;
    }
    async loadModels() {
        await Promise.all([
            this.load("asset", this.assetModels),
            this.load("device", this.deviceModels),
            this.load("group", this.groupModels),
            this.load("measure", this.measureModels),
        ]);
        await this.sdk.collection.refresh(this.config.adminIndex, plugin_1.InternalCollection.MODELS);
    }
    /**
     * Registers an asset model.
     *
     * @param engineGroup - The engine group name.
     * @param model - The name of the asset model, which must be in PascalCase.
     * @param measures - The measures associated with this asset model.
     * @param metadataMappings - The metadata mappings for the model, defaults to an empty object.
     * @param defaultMetadata - The default metadata values for the model, defaults to an empty object.
     * @param metadataDetails - Optional detailed metadata descriptions, localizations and definition.
     * @param metadataGroups - Optional groups for organizing metadata, with localizations.
     * @param tooltipModels - Optional model list for tooltip, containing labels and tooltip content.
     * @throws PluginImplementationError if the model name is not in PascalCase.
     */
    registerAsset(engineGroup, model, measures, metadataMappings = {}, defaultMetadata = {}, metadataDetails = {}, metadataGroups = {}, tooltipModels = {}, locales = {}) {
        if (kuzzle_1.Inflector.pascalCase(model) !== model) {
            throw new kuzzle_1.PluginImplementationError(`Asset model "${model}" must be PascalCase`);
        }
        const duplicates = (0, MeasuresDuplicates_1.getNamedMeasuresDuplicates)(measures);
        if (duplicates.length > 0) {
            throw new MeasuresNamesDuplicatesError_1.MeasuresNamesDuplicatesError("Asset model measures contain one or multiple duplicate measure name", duplicates);
        }
        // Construct and push the new asset model to the assetModels array
        this.assetModels.push({
            asset: {
                defaultMetadata,
                locales,
                measures,
                metadataDetails,
                metadataGroups,
                metadataMappings,
                model,
                tooltipModels,
            },
            engineGroup,
            type: "asset",
        });
    }
    /**
     * Registers a device model.
     *
     * @param model - The name of the device model, which must be in PascalCase.
     * @param measures - The measures associated with this device model.
     * @param metadataMappings - The metadata mappings for the model, defaults to an empty object.
     * @param defaultMetadata - The default metadata values for the model, defaults to an empty object.
     * @param metadataDetails - Optional detailed metadata descriptions, localizations and definition.
     * @param metadataGroups - Optional groups for organizing metadata, with localizations.
     * @throws PluginImplementationError if the model name is not in PascalCase.
     */
    registerDevice(model, measures, metadataMappings = {}, defaultMetadata = {}, metadataDetails = {}, metadataGroups = {}) {
        if (kuzzle_1.Inflector.pascalCase(model) !== model) {
            throw new kuzzle_1.PluginImplementationError(`Device model "${model}" must be PascalCase`);
        }
        const duplicates = (0, MeasuresDuplicates_1.getNamedMeasuresDuplicates)(measures);
        if (duplicates.length > 0) {
            throw new MeasuresNamesDuplicatesError_1.MeasuresNamesDuplicatesError("Device model measures contain one or multiple duplicate measure name", duplicates);
        }
        // Construct and push the new device model to the deviceModels array
        this.deviceModels.push({
            device: {
                defaultMetadata,
                measures,
                metadataDetails,
                metadataGroups,
                metadataMappings,
                model,
            },
            type: "device",
        });
    }
    /**
     * Registers a group model.
     *
     *
     * @param engineGroup - The engine group name.
     * @param model - The name of the group model, which must be in PascalCase.
     * @param metadataMappings - The metadata mappings for the model, defaults to an empty object.
     * @param defaultMetadata - The default metadata values for the model, defaults to an empty object.
     * @param metadataDetails - Optional detailed metadata descriptions, localizations and definition.
     * @param metadataGroups - Optional groups for organizing metadata, with localizations.
     * @throws PluginImplementationError if the model name is not in PascalCase.
     */
    registerGroup(engineGroup, model, metadataMappings = {}, defaultMetadata = {}, metadataDetails = {}, metadataGroups = {}) {
        if (kuzzle_1.Inflector.pascalCase(model) !== model) {
            throw new kuzzle_1.PluginImplementationError(`Group model "${model}" must be PascalCase`);
        }
        // Construct and push the new group model to the groupModels array
        this.groupModels.push({
            engineGroup,
            group: {
                defaultMetadata,
                metadataDetails,
                metadataGroups,
                metadataMappings,
                model,
            },
            type: "group",
        });
    }
    registerMeasure(type, measureDefinition) {
        const { locales, validationSchema, valuesMappings, valuesDetails } = measureDefinition;
        if (validationSchema) {
            try {
                (0, AJValidator_1.addSchemaToCache)(type, validationSchema);
            }
            catch (error) {
                throw new SchemaValidationError_1.SchemaValidationError("Provided schema is not valid", (0, AJValidator_1.getAJVErrors)());
            }
        }
        this.measureModels.push({
            measure: {
                locales,
                type,
                validationSchema,
                valuesDetails,
                valuesMappings,
            },
            type: "measure",
        });
    }
    async load(type, models) {
        const documents = models.map((model) => {
            return {
                _id: ModelSerializer_1.ModelSerializer.id(type, model),
                body: model,
            };
        });
        const modelTitles = models.map((model) => ModelSerializer_1.ModelSerializer.title(type, model));
        await this.sdk.document.mCreateOrReplace(this.config.adminIndex, plugin_1.InternalCollection.MODELS, documents, { strict: true });
        this.context.log.info(`Successfully load "${type}" models: ${modelTitles.join(", ")}`);
    }
}
exports.ModelsRegister = ModelsRegister;
//# sourceMappingURL=ModelsRegister.js.map