"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeviceManagerEngine = void 0;
const kuzzle_1 = require("kuzzle");
const kuzzle_plugin_commons_1 = require("kuzzle-plugin-commons");
const lodash_1 = __importDefault(require("lodash"));
const asset_1 = require("../asset");
const measure_1 = require("../measure");
const InternalCollection_1 = require("./types/InternalCollection");
const ModelsConflicts_1 = require("../model/ModelsConflicts");
const AJValidator_1 = require("../shared/utils/AJValidator");
class DeviceManagerEngine extends kuzzle_plugin_commons_1.AbstractEngine {
    get app() {
        return global.app;
    }
    constructor(plugin, adminConfigManager, engineConfigManager) {
        super("device-manager", plugin, plugin.config.adminIndex, adminConfigManager, engineConfigManager);
        this.context = plugin.context;
        (0, kuzzle_plugin_commons_1.onAsk)("ask:device-manager:engine:list", async ({ group }) => this.list(group));
        (0, kuzzle_plugin_commons_1.onAsk)("ask:device-manager:engine:updateAll", async () => {
            await this.updateEngines();
            await this.updateMeasuresSchema();
            await this.createDevicesCollection(this.config.adminIndex);
        });
        (0, kuzzle_plugin_commons_1.onAsk)("ask:device-manager:engine:doesUpdateConflict", async (payload) => {
            if (!payload) {
                return [];
            }
            if (payload.twin === undefined &&
                payload.measuresModels === undefined &&
                payload.groupModels === undefined) {
                return [];
            }
            const measureModels = await this.getModels(this.config.adminIndex, "measure");
            const results = await this.getEngines();
            const engineGroups = results.map((elt) => elt.engine.group);
            const groups = [undefined, ...engineGroups];
            for (const group of groups) {
                if (payload.twin) {
                    const twinModels = await this.getModels(this.config.adminIndex, payload.twin.type, group);
                    const conflicts = await this.doesTwinUpdateConflicts(payload.twin.type, twinModels, payload.twin.models, measureModels);
                    if (conflicts.length > 0) {
                        return conflicts;
                    }
                }
            }
            if (payload.measuresModels) {
                return this.doesMeasuresUpdateConflicts(measureModels, payload.measuresModels);
            }
            if (payload.groupModels) {
                const groupModels = await this.getModels(this.config.adminIndex, "group");
                return this.doesGroupsUpdateConflicts(groupModels, payload.groupModels);
            }
            return [];
        });
    }
    /**
     * Return conflicts between the new and already present twin models
     *
     * @param twinType The target twin type
     * @param twinModels The already present twin models
     * @param additionalModels The new or updated twin models
     * @param measureModels The already present measures models
     *
     * @throws If a measure type declared in the twin does not exists
     * @returns An array of conflict chunk
     */
    async doesTwinUpdateConflicts(twinType, twinModels, additionalModels, measureModels) {
        const conflicts = [];
        for (const additional of additionalModels) {
            for (const { type: measureType } of additional[twinType]
                .measures) {
                const measureModel = measureModels.find((m) => m.measure.type === measureType);
                if (!measureModel) {
                    throw new kuzzle_1.InternalError(`Cannot find measure "${measureType}" declared in ${[
                        twinType,
                    ]} "${additional[twinType].model}"`);
                }
            }
            conflicts.push(...(0, ModelsConflicts_1.getTwinConflicts)(twinType, twinModels, additional));
        }
        return conflicts;
    }
    /**
     * Return conflicts between the new and already present group models
     *
     * @param groupModels The already present group models
     * @param additionalGroups The new or updated group models
     * @returns An array of ConflictChunk
     */
    async doesGroupsUpdateConflicts(groupModels, additionalGroups) {
        const conflicts = [];
        for (const model of additionalGroups) {
            conflicts.push(...(0, ModelsConflicts_1.getGroupConflicts)(groupModels, model));
        }
        return conflicts;
    }
    /**
     * Return conflicts between the new and already present measure models
     *
     * @param measuresModels The already present measure models
     * @param additionalMeasures The new or updated measure models
     * @returns An array of ConflictChunk
     */
    async doesMeasuresUpdateConflicts(measuresModels, additionalMeasures) {
        const conflicts = [];
        for (const model of additionalMeasures) {
            conflicts.push(...(0, ModelsConflicts_1.getMeasureConflicts)(measuresModels, model));
        }
        return conflicts;
    }
    async updateEngines() {
        const results = await this.getEngines();
        this.context.log.info(`Update ${results.length} existing engines`);
        for (const document of results) {
            await this.onUpdate(document.engine.index, document.engine.group);
        }
    }
    /**
     * Search and return every engines in use
     *
     * @returns An array of the available engines
     */
    async getEngines() {
        const result = await this.sdk.document.search(this.config.adminIndex, InternalCollection_1.InternalCollection.CONFIG, {
            query: {
                equals: { type: "engine-device-manager" },
            },
        }, { lang: "koncorde", size: 5000 });
        return result.hits.map((elt) => elt._source);
    }
    async updateMeasuresSchema() {
        const models = await this.getModels(this.adminIndex, "measure");
        for (const measure of models) {
            const { validationSchema } = measure.measure;
            if (validationSchema) {
                try {
                    (0, AJValidator_1.addSchemaToCache)(measure.measure.type, validationSchema);
                }
                catch (error) {
                    this.app.log.error(`The validation schema for the "${measure.type}" measure model does not comply with the JSON Schema standard, so its update has been skipped`);
                }
            }
        }
    }
    async onCreate(index, group = "commons") {
        const promises = [];
        promises.push(this.createAssetsCollection(index, group));
        promises.push(this.createAssetsHistoryCollection(index, group));
        promises.push(this.createAssetsGroupsCollection(index));
        promises.push(this.createDevicesCollection(index));
        promises.push(this.createMeasuresCollection(index, group));
        const collections = await Promise.all(promises);
        return {
            collections: [this.engineConfigManager.collection, ...collections],
        };
    }
    async onUpdate(index, group = "commons") {
        const promises = [];
        promises.push(this.createAssetsCollection(index, group));
        promises.push(this.createAssetsHistoryCollection(index, group));
        promises.push(this.createAssetsGroupsCollection(index));
        promises.push(this.createDevicesCollection(index));
        promises.push(this.createMeasuresCollection(index, group));
        const collections = await Promise.all(promises);
        return { collections };
    }
    async onDelete(index) {
        const collections = [
            InternalCollection_1.InternalCollection.ASSETS,
            InternalCollection_1.InternalCollection.ASSETS_HISTORY,
            InternalCollection_1.InternalCollection.ASSETS_GROUPS,
            InternalCollection_1.InternalCollection.DEVICES,
            InternalCollection_1.InternalCollection.MEASURES,
        ];
        await Promise.all(collections.map(async (collection) => {
            if (await this.sdk.collection.exists(index, collection)) {
                await this.sdk.collection.delete(index, collection);
            }
        }));
        await this.detachDevicesFromAdminIndex(index);
        return {
            collections,
        };
    }
    /**
     * Generate assets mappings and create the assets collection in the engine
     *
     * @param engineId The target engine Id
     * @param engineGroup The engine group
     *
     * @throws If it failed during the assets collection creation
     */
    async createAssetsCollection(engineId, engineGroup) {
        const mappings = await this.getDigitalTwinMappingsFromDB("asset", engineGroup);
        const settings = this.config.engineCollections.asset.settings;
        await this.tryCreateCollection(engineId, InternalCollection_1.InternalCollection.ASSETS, {
            mappings,
            settings,
        });
        return InternalCollection_1.InternalCollection.ASSETS;
    }
    /**
     * Generate assets mappings and create the assets history collection in the engine
     *
     * @param engineId The target engine Id
     * @param engineGroup The engine group
     *
     * @throws If it failed during the assets history collection creation
     */
    async createAssetsHistoryCollection(engineId, engineGroup) {
        const assetsMappings = await this.getDigitalTwinMappingsFromDB("asset", engineGroup);
        const mappings = JSON.parse(JSON.stringify(asset_1.assetsHistoryMappings));
        lodash_1.default.merge(mappings.properties.asset, assetsMappings);
        const settings = this.config.engineCollections.assetHistory.settings;
        await this.tryCreateCollection(engineId, InternalCollection_1.InternalCollection.ASSETS_HISTORY, {
            mappings,
            settings,
        });
        return InternalCollection_1.InternalCollection.ASSETS_HISTORY;
    }
    /**
     * Create the assets groups collection with the assets groups mappings in the engine
     *
     * @param engineId The target engine Id
     * @param engineGroup The engine group
     *
     * @throws If it failed during the assets groups collection creation
     */
    async createAssetsGroupsCollection(engineId) {
        const settings = this.config.engineCollections.assetGroups.settings;
        const mappings = await this.getAssetGroupsMappingFromDB();
        await this.tryCreateCollection(engineId, InternalCollection_1.InternalCollection.ASSETS_GROUPS, {
            mappings,
            settings,
        });
        return InternalCollection_1.InternalCollection.ASSETS_GROUPS;
    }
    /**
     * Generate devices mappings and create the devices collection in the engine
     *
     * @param engineId The target engine Id
     * @param engineGroup The engine group
     *
     * @throws If it failed during the devices collection creation
     */
    async createDevicesCollection(engineId) {
        const mappings = await this.getDigitalTwinMappingsFromDB("device");
        const settings = this.config.engineCollections.device.settings;
        await this.tryCreateCollection(engineId, InternalCollection_1.InternalCollection.DEVICES, {
            mappings,
            settings,
        });
        return InternalCollection_1.InternalCollection.DEVICES;
    }
    /**
     * Generate measures mappings and create the measures collection in the engine
     *
     * @param engineId The target engine Id
     * @param engineGroup The engine group
     *
     * @throws If it failed during the measures collection creation
     */
    async createMeasuresCollection(engineId, engineGroup) {
        const mappings = await this.getMeasuresMappingsFromDB(engineGroup);
        const settings = this.config.engineCollections.measures.settings;
        await this.tryCreateCollection(engineId, InternalCollection_1.InternalCollection.MEASURES, {
            mappings,
            settings,
        });
        return InternalCollection_1.InternalCollection.MEASURES;
    }
    /**
     * Create a collection with custom mappings in an engine
     *
     * @param engineIndex The target engine
     * @param collection The collection name
     * @param mappings The collection mappings
     *
     * @throws If it failed to create the collection
     */
    async tryCreateCollection(engineIndex, collection, options) {
        try {
            await this.sdk.collection.create(engineIndex, collection, options);
        }
        catch (error) {
            throw new kuzzle_1.InternalError(`Failed to create the collection [${collection}] for engine [${engineIndex}]: ${error.message}`, error);
        }
    }
    /**
     * Generate ES mappings from twin models and theirs associated measures, all fetched from the database
     *
     * @param digitalTwinType The target twin type
     * @param engineGroup The twin engine group
     * @returns The complete ES mappings produces by merging all the target type twins
     */
    async getDigitalTwinMappingsFromDB(digitalTwinType, engineGroup) {
        const models = await this.getModels(this.config.adminIndex, digitalTwinType, engineGroup);
        const measureModels = await this.getModels(this.config.adminIndex, "measure");
        return this.getDigitalTwinMappings(digitalTwinType, models, measureModels);
    }
    /**
     * Generate ES mappings from twin models and theirs associated measures
     *
     * @param digitalTwinType The target twin type
     * @param models The twin models
     * @param measureModels The associated measures
     * @returns The complete ES mappings produces by merging all the target type twins
     */
    async getDigitalTwinMappings(digitalTwinType, models, measureModels) {
        if (this.config.engineCollections[digitalTwinType] === undefined ||
            this.config.engineCollections[digitalTwinType].mappings === undefined) {
            throw new kuzzle_1.InternalError(`Cannot find mapping for "${digitalTwinType}"`);
        }
        const mappings = JSON.parse(JSON.stringify(this.config.engineCollections[digitalTwinType].mappings));
        for (const model of models) {
            lodash_1.default.merge(mappings.properties.metadata.properties, model[digitalTwinType].metadataMappings);
            for (const { name: measureName, type: measureType } of model[digitalTwinType].measures) {
                const measureModel = measureModels.find((m) => m.measure.type === measureType);
                if (!measureModel) {
                    throw new kuzzle_1.InternalError(`Cannot find measure "${measureType}" declared in ${[
                        digitalTwinType,
                    ]} "${model[digitalTwinType].model}"`);
                }
                mappings.properties.measures.properties[measureName] =
                    (0, measure_1.getEmbeddedMeasureMappings)(measureModel.measure.valuesMappings);
            }
        }
        return mappings;
    }
    /**
     * Generate ES mappings from measures and theirs associated assets, all fetched via the database
     *
     * @param engineGroup The target engine group
     * @returns The complete ES mappings produced by merging models mappings
     */
    async getMeasuresMappingsFromDB(engineGroup) {
        const models = await this.getModels(this.config.adminIndex, "measure");
        const assetsMappings = await this.getDigitalTwinMappingsFromDB("asset", engineGroup);
        const deviceMappings = await this.getDigitalTwinMappingsFromDB("device");
        return this.getMeasuresMappings(models, assetsMappings, deviceMappings);
    }
    /**
     * Generate ES mappings from measures and theirs associated assets
     *
     * @param models The measures models
     * @param assetsMappings The assets complete mappings
     * @returns The complete ES mappings produced by merging models mappings
     */
    async getMeasuresMappings(models, assetsMappings, deviceMappings) {
        const mappings = JSON.parse(JSON.stringify(measure_1.measuresMappings));
        for (const model of models) {
            lodash_1.default.merge(mappings.properties.values.properties, model.measure.valuesMappings);
        }
        mappings.properties.asset.properties.metadata.properties =
            assetsMappings.properties.metadata.properties;
        mappings.properties.origin.properties.deviceMetadata.properties =
            deviceMappings.properties.metadata.properties;
        return mappings;
    }
    async getAssetGroupsMappingFromDB() {
        const models = await this.getModels(this.config.adminIndex, "group");
        const mappings = JSON.parse(JSON.stringify(this.config.engineCollections.assetGroups.mappings));
        for (const model of models) {
            lodash_1.default.merge(mappings.properties.metadata.properties, model.group.metadataMappings);
        }
        return mappings;
    }
    /**
     * Retrieve a certain type of models associated to an engine
     *
     * @param engineId The target engine Id
     * @param type The desired model type
     * @param engineGroup The target engine group
     * @returns An array of the generic type provided
     */
    async getModels(engineId, type, engineGroup) {
        const query = {
            and: [{ equals: { type } }],
        };
        if (engineGroup) {
            query.and.push({
                or: [
                    { equals: { engineGroup } },
                    { equals: { engineGroup: "commons" } },
                ],
            });
        }
        const result = await this.sdk.document.search(engineId, InternalCollection_1.InternalCollection.MODELS, { query }, { lang: "koncorde", size: 5000 });
        return result.hits.map((elt) => elt._source);
    }
    /**
     * Detach all devices of an index in the admin index
     *
     * @param engineId The target engine Id
     * @returns {any}
     */
    async detachDevicesFromAdminIndex(engineId) {
        const devices = [];
        let result = await this.sdk.document.search(this.adminIndex, "devices", {
            _source: false,
            query: { bool: { must: { term: { engineId } } } },
        }, {
            scroll: "2s",
            size: 100,
        });
        while (result !== null) {
            devices.push(...result.hits);
            result = await result.next();
        }
        if (devices.length > 0) {
            void this.sdk.document.mUpdate(this.adminIndex, "devices", devices.map((device) => {
                return { _id: device._id, body: { assetId: null, engineId: null } };
            }));
        }
    }
}
exports.DeviceManagerEngine = DeviceManagerEngine;
//# sourceMappingURL=DeviceManagerEngine.js.map