"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeviceManagerPlugin = void 0;
const kuzzle_1 = require("kuzzle");
const kuzzle_plugin_commons_1 = require("kuzzle-plugin-commons");
const lodash_1 = __importDefault(require("lodash"));
const measure_1 = require("../measure");
const asset_1 = require("../asset");
const decoder_1 = require("../decoder");
const device_1 = require("../device");
const measure_2 = require("../measure");
const model_1 = require("../model");
const shared_1 = require("../shared");
const DeviceManagerEngine_1 = require("./DeviceManagerEngine");
const InternalCollection_1 = require("./types/InternalCollection");
class DeviceManagerPlugin extends kuzzle_1.Plugin {
    get sdk() {
        return this.context.accessors.sdk;
    }
    get models() {
        return {
            /**
             * Register an asset model
             *
             * @param engineGroup Engine group name
             * @param model Name of the asset model. Must follow a naming convention in PascalCase.
             * @param definition Object containing the asset model definition, including:
             *        - measures: Array describing measure names and their types.
             *        - metadataMappings: Definition of metadata mappings, specifying types for each metadata field.
             *        - defaultMetadata: Default values for metadata fields, applied when actual data is not provided.
             *        - metadataDetails: Optional detailed descriptions for each metadata, including localizations, group association and definition.
             *        - metadataGroups: Optional description of metadata groups, organizing metadata logically, with localizations for group names.
             *        - tooltipModels: Optional tooltip model list, containing each labels and tooltip content to display.
             *
             * @example
             * ```
             * deviceManager.models.registerAsset(
             *   "logistic",
             *   "Container",
             *   {
             *     measures: [
             *       { name: "temperatureExt", type: "temperature" },
             *       { name: "temperatureInt", type: "temperature" },
             *       { name: "position", type: "position" },
             *     ],
             *     metadataMappings: {
             *       weight: { type: "integer" },
             *       height: { type: "integer" },
             *     },
             *     defaultMetadata: {
             *       height: 20
             *     },
             *     metadataDetails: {
             *       "extTemp": {
             *         "group": "environment",
             *         "locales": {
             *           "en": {
             *             "friendlyName": "External Temperature",
             *             "description": "The temperature outside the container"
             *           },
             *           "fr": {
             *             "friendlyName": "Température Externe",
             *             "description": "La température à l'extérieur du conteneur"
             *           }
             *         }
             *       }
             *     },
             *     metadataGroups: {
             *       "environment": {
             *         "locales": {
             *           "en": {
             *             "groupFriendlyName": "Environment"
             *           },
             *           "fr": {
             *             "groupFriendlyName": "Environnement"
             *           }
             *         }
             *       }
             *     },
             *     tooltipModels: {
             *       "defaultTooltipKey": {
             *         "tooltipLabel": "Default Tooltip Model",
             *         "content": [
             *           {
             *             "category": "metadata",
             *             "label": {
             *               "locales": {
             *                 "en": {
             *                   "friendlyName": "Container position",
             *                   "description": ""
             *                 },
             *                 "fr": {
             *                   "friendlyName": "Position du conteneur",
             *                   "description": ""
             *                 }
             *               }
             *             },
             *             "metadataPath": "geolocation"
             *           },
             *           {
             *             "category": "measure",
             *             "label": {
             *               "locales": {
             *                 "en": {
             *                   "friendlyName": "External temperature",
             *                   "description": ""
             *                 },
             *                 "fr": {
             *                   "friendlyName": "Température extérieure",
             *                   "description": ""
             *                 }
             *               }
             *             },
             *             "measureSlot": "externalTemperature",
             *             "measureValuePath": "externalTemperature",
             *             "suffix": "°C"
             *           }
             *         ]
             *       }
             *     }
             *   }
             * );
             * ```
             */
            registerAsset: (engineGroup, model, definition) => {
                this.modelsRegister.registerAsset(engineGroup, model, definition.measures, definition.metadataMappings, definition.defaultMetadata, definition.metadataDetails, definition.metadataGroups, definition.tooltipModels, definition.locales);
            },
            /**
             * Register a device.
             *
             * @param model Name of the device model
             * @param definition Object containing the device model definition, including:
             *                   - decoder: Decoder used to decode payloads
             *                   - metadataMappings: Metadata mappings definition
             *                   - defaultMetadata: Default metadata values
             *                   - metadataDetails: Localizations, detailed metadata descriptions and definition
             *                   - metadataGroups: Groups for organizing metadata, with localizations
             *
             * @example
             * ```
             * deviceManager.models.registerDevice(
             *   "Abeeway",
             *   {
             *     decoder: new DummyTempPositionDecoder(),
             *     metadataMappings: {
             *       serial: { type: "keyword" },
             *     },
             *     defaultMetadata: {
             *       company: "Acme Inc"
             *     },
             *     metadataDetails: {
             *       sensorVersion: {
             *         group: "sensorSpecs",
             *         locales: {
             *           en: {
             *             friendlyName: "Sensor version",
             *             description: "Firmware version of the sensor"
             *           },
             *           fr: {
             *             friendlyName: "Version du capteur",
             *             description: "Version du micrologiciel du capteur"
             *           }
             *         }
             *       }
             *     },
             *     metadataGroups: {
             *       sensorSpecs: {
             *         locales: {
             *           en: {
             *             groupFriendlyName: "Sensors specifications"
             *           },
             *           fr: {
             *             groupFriendlyName: "Spécifications des capteurs"
             *           }
             *         }
             *       }
             *     }
             *   }
             * );
             * ```
             */
            registerDevice: (model, definition) => {
                this.decodersRegister.register(definition.decoder);
                definition.decoder.log = this.context?.log;
                this.modelsRegister.registerDevice(model, definition.decoder.measures, definition.metadataMappings, definition.defaultMetadata, definition.metadataDetails, definition.metadataGroups);
            },
            /**
             * Register a group.
             *
             * @param model Name of the group model
             * @param definition Object containing the group model definition, including:
             *                   - metadataMappings: Metadata mappings definition
             *                   - defaultMetadata: Default metadata values
             *                   - metadataDetails: Localizations, detailed metadata descriptions and definition
             *                   - metadataGroups: Groups for organizing metadata, with localizations
             *
             * @example
             * ```
             * deviceManager.models.registerGroup(
             *   "Parking",
             *   {
             *     metadataMappings: {
             *       geolocation: { type: "geo_point" },
             *     },
             *     defaultMetadata: {
             *
             *     },
             *     metadataDetails: {
             *       geolocation: {
             *         group: "access",
             *         locales: {
             *           en: {
             *             friendlyName: "Parking location",
             *             description: "GPS position of the parking"
             *           },
             *           fr: {
             *             friendlyName: "Emplacement du parking",
             *             description: "Position GPS du parking"
             *           }
             *         }
             *       }
             *     },
             *     metadataGroups: {
             *       access: {
             *         locales: {
             *           en: {
             *             groupFriendlyName: "Parking access info"
             *           },
             *           fr: {
             *             groupFriendlyName: "Information d'accès au parking"
             *           }
             *         }
             *       }
             *     }
             *   }
             * );
             * ```
             */
            registerGroup: (engineGroup, model, definition) => {
                this.modelsRegister.registerGroup(engineGroup, model, definition.metadataMappings, definition.defaultMetadata, definition.metadataDetails, definition.metadataGroups);
            },
            /**
             * Register a new measure
             *
             * @param name Name of the measure
             * @param measureDefinition Values of the measure
             *
             * @example
             * ```
             * deviceManager.models.registerMeasure("acceleration", {
             *   x: { type: "float" },
             *   y: { type: "float" },
             *   z: { type: "float" },
             * });
             * ```
             */
            registerMeasure: (name, measureDefinition) => {
                this.modelsRegister.registerMeasure(name, measureDefinition);
            },
        };
    }
    constructor() {
        super({
            kuzzleVersion: ">=2.20.2 <3",
        });
        /* eslint-disable sort-keys */
        this.api = {};
        this.pipes = {
            "generic:document:beforeWrite": [],
            "generic:document:beforeUpdate": [],
            "generic:document:beforeDelete": [],
        };
        this.hooks = {};
        this.imports = {
            roles: {},
            users: {},
            profiles: {},
            onExistingUsers: "skip",
        };
        this.config = {
            ignoreStartupErrors: false,
            assetsHistorizesMeasures: false,
            engine: {
                autoUpdate: true,
            },
            adminIndex: "device-manager",
            adminCollections: {
                config: {
                    name: "config",
                    mappings: {
                        dynamic: "strict",
                        properties: {},
                    },
                },
                devices: {
                    name: "devices",
                    mappings: device_1.devicesMappings,
                },
                payloads: {
                    name: "payloads",
                    mappings: decoder_1.payloadsMappings,
                },
            },
            engineCollections: {
                config: {
                    name: "config",
                    mappings: {
                        dynamic: "strict",
                        properties: {},
                    },
                },
                asset: {
                    name: InternalCollection_1.InternalCollection.ASSETS,
                    mappings: asset_1.assetsMappings,
                },
                assetGroups: {
                    name: InternalCollection_1.InternalCollection.ASSETS_GROUPS,
                    mappings: asset_1.assetGroupsMappings,
                },
                assetHistory: {
                    name: InternalCollection_1.InternalCollection.ASSETS_HISTORY,
                },
                device: {
                    name: InternalCollection_1.InternalCollection.DEVICES,
                    mappings: device_1.devicesMappings,
                },
                measures: {
                    name: InternalCollection_1.InternalCollection.MEASURES,
                },
            },
        };
        /* eslint-enable sort-keys */
        // Registers
        this.decodersRegister = new decoder_1.DecodersRegister();
        this.modelsRegister = new model_1.ModelsRegister();
        this.models.registerMeasure("temperature", measure_1.temperatureMeasureDefinition);
        this.models.registerMeasure("position", measure_1.positionMeasureDefinition);
        this.models.registerMeasure("movement", measure_1.movementMeasureDefinition);
        this.models.registerMeasure("humidity", measure_1.humidityMeasureDefinition);
        this.models.registerMeasure("battery", measure_1.batteryMeasureDefinition);
    }
    /**
     * Init the plugin
     */
    async init(config, context) {
        this.config = lodash_1.default.merge({}, this.config, config);
        this.context = context;
        for (const decoder of this.decodersRegister.decoders) {
            decoder.log = this.context.log;
        }
        // Modules creation
        this.assetModule = new asset_1.AssetModule(this);
        this.deviceModule = new device_1.DeviceModule(this);
        this.decoderModule = new decoder_1.DecoderModule(this);
        this.measureModule = new measure_2.MeasureModule(this);
        this.modelModule = new model_1.ModelModule(this);
        // Modules init
        await this.assetModule.init();
        await this.deviceModule.init();
        await this.decoderModule.init();
        await this.measureModule.init();
        await this.modelModule.init();
        this.decodersRegister.init(this, this.context);
        this.modelsRegister.init(this);
        this.adminConfigManager = new kuzzle_plugin_commons_1.ConfigManager(this, {
            mappings: this.config.adminCollections.config.mappings,
            settings: this.config.adminCollections.config.settings,
        });
        this.adminConfigManager.register("device-manager", {
            properties: {
                provisioningStrategy: { type: "keyword" },
            },
        });
        this.adminConfigManager.register("engine", {
            properties: {
                group: { type: "keyword" },
                index: { type: "keyword" },
                name: { type: "keyword" },
            },
        });
        this.engineConfigManager = new kuzzle_plugin_commons_1.ConfigManager(this, {
            mappings: this.config.engineCollections.config.mappings,
            settings: this.config.engineCollections.config.settings,
        });
        this.deviceManagerEngine = new DeviceManagerEngine_1.DeviceManagerEngine(this, this.adminConfigManager, this.engineConfigManager);
        this.engineController = new kuzzle_plugin_commons_1.EngineController("device-manager", this, this.deviceManagerEngine);
        this.decodersRegister.registerDefaultRights();
        this.decodersRegister.printDecoders();
        try {
            await this.initDatabase();
        }
        catch (error) {
            if (this.config.ignoreStartupErrors) {
                this.context.log.warn(`WARNING: An error occured during plugin initialization: ${error.message}`);
            }
            else {
                throw error;
            }
        }
        if (this.config.ignoreStartupErrors) {
            this.context.log.warn('WARNING: The "ignoreStartupErrors" option is enabled. Additional errors may appears at runtime.');
        }
    }
    /**
     * Initialize the administration index of the plugin
     */
    async initDatabase() {
        await (0, shared_1.lock)("device-manager/initDatabase", async () => {
            if (!(await this.sdk.index.exists(this.config.adminIndex))) {
                // Possible race condition because of index cache propagation.
                // The index has been created but the node didn't receive the index
                // cache update message yet, causing index:exists to returns false
                try {
                    await this.sdk.index.create(this.config.adminIndex);
                }
                catch (error) {
                    if (!error.message.includes("already exists")) {
                        throw error;
                    }
                }
            }
            await this.adminConfigManager
                .createCollection(this.config.adminIndex)
                .catch((error) => {
                throw (0, shared_1.keepStack)(error, new kuzzle_1.PluginImplementationError(`Cannot create admin "config" collection: ${error}`));
            });
            await this.sdk.collection
                .create(this.config.adminIndex, InternalCollection_1.InternalCollection.MODELS, {
                mappings: model_1.modelsMappings,
            })
                .catch((error) => {
                throw (0, shared_1.keepStack)(error, new kuzzle_1.PluginImplementationError(`Cannot create admin "models" collection: ${error}`));
            });
            await this.modelsRegister.loadModels();
            await this.deviceManagerEngine
                .createDevicesCollection(this.config.adminIndex)
                .catch((error) => {
                throw (0, shared_1.keepStack)(error, new kuzzle_1.PluginImplementationError(`Cannot create admin "devices" collection: ${error}`));
            });
            await this.sdk.collection
                .create(this.config.adminIndex, "payloads", {
                mappings: this.getPayloadsMappings(),
                settings: this.config.adminCollections.payloads.settings,
            })
                .catch((error) => {
                throw (0, shared_1.keepStack)(error, new kuzzle_1.PluginImplementationError(`Cannot create admin "payloads" collection: ${error}`));
            });
            await this.initializeConfig();
            if (this.config.engine.autoUpdate) {
                try {
                    await this.deviceManagerEngine.updateEngines();
                    await this.deviceManagerEngine.updateMeasuresSchema();
                }
                catch (error) {
                    this.context.log.error(`An error occured while updating the engines during startup: ${error}`);
                }
            }
        });
    }
    /**
     * Merge custom mappings defined in the Decoder into the "payloads" collection
     * mappings.
     *
     * Those custom mappings allow to search raw payloads more efficiently.
     */
    getPayloadsMappings() {
        const { mappings } = JSON.parse(JSON.stringify(this.config.adminCollections.payloads));
        for (const decoder of this.decodersRegister.decoders) {
            mappings.properties.payload.properties = {
                ...mappings.properties.payload.properties,
                ...decoder.payloadsMappings,
            };
        }
        return mappings;
    }
    /**
     * Initialize the config document if it does not exists
     */
    async initializeConfig() {
        const exists = await this.sdk.document.exists(this.config.adminIndex, this.adminConfigManager.collection, "plugin--device-manager");
        if (!exists) {
            await this.sdk.document.create(this.config.adminIndex, this.adminConfigManager.collection, {
                "device-manager": { provisioningStrategy: "auto" },
                type: "device-manager",
            }, "plugin--device-manager");
        }
    }
    async pipeCheckEngine(request) {
        const engineId = request.getString("engineId");
        if (engineId !== this.config.adminIndex) {
            const { result: { exists }, } = await this.sdk.query({
                action: "exists",
                controller: "device-manager/engine",
                index: engineId,
            });
            if (!exists) {
                throw new kuzzle_1.BadRequestError(`Tenant "${engineId}" does not have a device-manager engine`);
            }
        }
        return request;
    }
}
exports.DeviceManagerPlugin = DeviceManagerPlugin;
//# sourceMappingURL=DeviceManagerPlugin.js.map