"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractExporter = void 0;
const node_crypto_1 = require("node:crypto");
const node_stream_1 = require("node:stream");
const sync_1 = require("csv-stringify/sync");
const kuzzle_1 = require("kuzzle");
const lodash_1 = __importDefault(require("lodash"));
const plugin_1 = require("../../plugin");
class AbstractExporter {
    constructor(plugin, target, config = {}) {
        this.plugin = plugin;
        this.target = target;
        this.config = {
            expireTime: 2 * 60,
        };
        if (Object.keys(config).length > 0) {
            this.config = {
                ...this.config,
                ...config,
            };
        }
    }
    get sdk() {
        return this.plugin.context.accessors.sdk;
    }
    get ms() {
        return this.sdk.ms;
    }
    get log() {
        return this.plugin.context.log;
    }
    async getEngine(engineId) {
        const engine = await this.sdk.document.get(this.plugin.config.adminIndex, plugin_1.InternalCollection.CONFIG, `engine-device-manager--${engineId}`);
        return engine._source.engine;
    }
    async prepareExport(engineId, user, params) {
        const exportId = (0, node_crypto_1.randomUUID)();
        await this.ms.setex(this.exportRedisKey(engineId, exportId), JSON.stringify(params), this.config.expireTime);
        let link = this.getLink(engineId, exportId, params);
        if (user._id !== "-1") {
            const { result } = await this.sdk.as(user).query({
                action: "createToken",
                controller: "auth",
                expiresIn: this.config.expireTime * 1000,
                singleUse: true,
            });
            link += `?jwt=${result.token}`;
        }
        return link;
    }
    formatHit(columns, hit) {
        return columns.map(({ path, isIsoDate }) => {
            const formattedValue = lodash_1.default.get(hit, path, null);
            if (formattedValue !== null && isIsoDate) {
                return new Date(formattedValue).toISOString();
            }
            return lodash_1.default.get(hit, path, null);
        });
    }
    async getExport(engineId, exportId) {
        const exportParams = await this.sdk.ms.get(this.exportRedisKey(engineId, exportId));
        if (!exportParams) {
            throw new kuzzle_1.NotFoundError(`Export "${exportId}" not found or expired.`);
        }
        return JSON.parse(exportParams);
    }
    async getExportStream(request, columns) {
        const stream = new node_stream_1.PassThrough();
        let result = request;
        try {
            stream.write((0, sync_1.stringify)([columns.map((column) => column.header)]));
            while (result) {
                for (const hit of result.hits) {
                    stream.write((0, sync_1.stringify)([this.formatHit(columns, hit)]));
                }
                result = result.fetched < 10000 ? await result.next() : null;
            }
        }
        catch (error) {
            stream.write(error.message);
        }
        finally {
            stream.end();
        }
        return stream;
    }
}
exports.AbstractExporter = AbstractExporter;
//# sourceMappingURL=AbstractExporter.js.map