"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseService = void 0;
const kuzzle_1 = require("kuzzle");
class BaseService {
    constructor(plugin) {
        this.plugin = plugin;
    }
    get app() {
        return global.app;
    }
    get sdk() {
        return this.plugin.context.accessors.sdk;
    }
    get config() {
        return this.plugin.config;
    }
    get impersonatedSdk() {
        return (user) => {
            if (user?._id) {
                return this.sdk.as(user, { checkRights: false });
            }
            return this.sdk;
        };
    }
    normalizeKuzzleRequest(request, { collection, engineId }) {
        request.input.args.collection = collection;
        request.input.args.index = engineId;
        return request;
    }
    /**
     * Wrapper to SDK create method with trigger generic:document events
     *
     * @param {KuzzleRequest} request
     * @param {KDocument} documentId
     * @param {PayloadRequest} payload
     * @param {ArgsDocumentControllerCreate} [options]
     * @returns {Promise<KDocument>}
     */
    async getDocument(request, documentId, { collection, engineId }, options = {}) {
        const kuzzleRequest = this.normalizeKuzzleRequest(request, {
            collection,
            engineId,
        });
        const refresh = kuzzleRequest.getRefresh();
        const [{ _id }] = await this.app.trigger("generic:document:beforeGet", [{ _id: documentId }], kuzzleRequest);
        const newDocument = await this.sdk.document.get(engineId, collection, _id, { refresh, ...options });
        const [endDocument] = await this.app.trigger("generic:document:afterGet", [newDocument], kuzzleRequest);
        return endDocument;
    }
    /**
     * Wrapper to SDK create method with trigger generic:document events
     *
     * @param {KuzzleRequest} request
     * @param {KDocument} document
     * @param {PayloadRequest} payload
     * @param {ArgsDocumentControllerCreate} [options]
     * @returns {Promise<KDocument>}
     */
    async createDocument(request, document, { collection, engineId }, options = {}) {
        const kuzzleRequest = this.normalizeKuzzleRequest(request, {
            collection,
            engineId,
        });
        const user = kuzzleRequest.getUser();
        const refresh = kuzzleRequest.getRefresh();
        const [modifiedDocument] = await this.app.trigger("generic:document:beforeWrite", [document], kuzzleRequest);
        const newDocument = await this.impersonatedSdk(user).document.create(engineId, collection, modifiedDocument._source, modifiedDocument._id, { refresh, ...options });
        const [endDocument] = await this.app.trigger("generic:document:afterWrite", [newDocument], kuzzleRequest);
        return endDocument;
    }
    /**
     * Wrapper to SDK update method with trigger generic:document events
     *
     * @param {KuzzleRequest} request
     * @param {KDocument} document
     * @param {PayloadRequest} payload
     * @param {ArgsDocumentControllerUpdate} [options]
     * @returns {Promise<KDocument>}
     */
    async updateDocument(request, document, { collection, engineId }, options = {}) {
        const kuzzleRequest = this.normalizeKuzzleRequest(request, {
            collection,
            engineId,
        });
        const user = kuzzleRequest.getUser();
        const refresh = kuzzleRequest.getRefresh();
        const [modifiedDocument] = await this.app.trigger("generic:document:beforeUpdate", [document], kuzzleRequest);
        const updatedDocument = await this.impersonatedSdk(user).document.update(engineId, collection, modifiedDocument._id, modifiedDocument._source, { refresh, ...options });
        const [endDocument] = await this.app.trigger("generic:document:afterUpdate", [updatedDocument], kuzzleRequest);
        return endDocument;
    }
    /**
     * Wrapper to SDK delete method with trigger generic:document events
     *
     * @param {KuzzleRequest} request
     * @param {string} documentId
     * @param {PayloadRequest} payload
     * @param {ArgsDocumentControllerUpdate} [options]
     * @returns {Promise<{ _id: string }>}
     */
    async deleteDocument(request, documentId, { collection, engineId }, options = {}) {
        const kuzzleRequest = this.normalizeKuzzleRequest(request, {
            collection,
            engineId,
        });
        const user = kuzzleRequest.getUser();
        const refresh = kuzzleRequest.getRefresh();
        const [modifiedDocument] = await this.app.trigger("generic:document:beforeDelete", [{ _id: documentId }], kuzzleRequest);
        const deletedDocument = await this.impersonatedSdk(user).document.delete(engineId, collection, modifiedDocument._id, { refresh, ...options });
        const [endDocument] = await this.app.trigger("generic:document:afterDelete", [{ _id: deletedDocument }], kuzzleRequest);
        return endDocument;
    }
    /**
     * Wrapper to SDK search method with trigger generic:document events
     * ! Caution the pipes are not applied on next()
     *
     * @param {KuzzleRequest} request
     * @param {ArgsDocumentControllerUpdate} [options]
     * @param {PayloadRequest} payload
     * @returns {Promise<SearchResult<KHit>>}
     */
    async searchDocument(request, { from, size, scrollTTL: scroll }, { collection, engineId }) {
        const kuzzleRequest = this.normalizeKuzzleRequest(request, {
            collection,
            engineId,
        });
        const { protocol, misc: { verb = "POST" }, } = kuzzleRequest.context.connection;
        const lang = kuzzleRequest.getLangParam();
        const searchBody = kuzzleRequest.getSearchBody();
        const modifiedBody = await this.app.trigger("generic:document:beforeSearch", searchBody, kuzzleRequest);
        const query = {
            action: "search",
            body: null,
            collection,
            controller: "document",
            from,
            index: engineId,
            lang,
            scroll,
            searchBody: null,
            size,
            verb,
        };
        if (protocol === "http" && verb === "GET") {
            query.searchBody = modifiedBody;
        }
        else {
            query.body = modifiedBody;
        }
        const { result } = await this.sdk.query(query);
        const modifiedResult = await this.app.trigger("generic:document:afterSearch", result, request);
        return new kuzzle_1.DocumentSearchResult(global, query, {}, modifiedResult);
    }
}
exports.BaseService = BaseService;
//# sourceMappingURL=BaseService.js.map