"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DigitalTwinExporter = void 0;
const kuzzle_plugin_commons_1 = require("kuzzle-plugin-commons");
const __1 = require("../");
const plugin_1 = require("../../plugin");
const AbstractExporter_1 = require("./AbstractExporter");
class DigitalTwinExporter extends AbstractExporter_1.AbstractExporter {
    exportRedisKey(engineId, exportId) {
        return `exports:${engineId}:${this.target}:${exportId}`;
    }
    getLink(engineId, exportId) {
        return `/_/device-manager/${engineId}/${this.target}/_export/${exportId}`;
    }
    async sendExport(engineId, exportId) {
        const { query, sort, lang = "elasticsearch", } = await this.getExport(engineId, exportId);
        const digitalTwins = await this.sdk.document.search(engineId, this.target, { query, sort }, { lang, size: 200 });
        const namedMeasures = await this.getNamedMeasures(engineId);
        const measureColumns = await this.generateMeasureColumns(namedMeasures);
        const columns = [
            { header: "Model", path: "_source.model" },
            { header: "Reference", path: "_source.reference" },
            ...measureColumns,
            { header: "lastMeasuredAt", path: "_source.lastMeasuredAt" },
            {
                header: "lastMeasuredAtISO",
                isIsoDate: true,
                path: "_source.lastMeasuredAt",
            },
        ];
        const stream = this.getExportStream(digitalTwins, columns);
        await this.sdk.ms.del(this.exportRedisKey(engineId, exportId));
        return stream;
    }
    /**
     * Get the deduplicated Named Measures get from models
     */
    async getNamedMeasures(engineId) {
        const type = this.target === plugin_1.InternalCollection.ASSETS ? "asset" : "device";
        const query = {
            and: [{ equals: { type } }],
        };
        if (this.target === plugin_1.InternalCollection.ASSETS) {
            const engine = await this.getEngine(engineId);
            query.and.push({ equals: { engineGroup: engine.group } });
        }
        let result = await this.sdk.document.search(this.plugin.config.adminIndex, plugin_1.InternalCollection.MODELS, {
            query: { equals: { type } },
        }, { lang: "koncorde", size: 20 });
        // ? Use a map to dedup the NamedMeasures get from models
        const namedMeasures = new Map();
        while (result) {
            for (const { _source } of result.hits) {
                for (const namedMeasure of _source[type].measures) {
                    if (!namedMeasures.has(namedMeasure.name)) {
                        namedMeasures.set(namedMeasure.name, namedMeasure);
                    }
                }
            }
            result = await result.next();
        }
        // ? Ensure stable measures order
        return [...namedMeasures.values()].sort((a, b) => {
            if (a.name < b.name) {
                return -1;
            }
            if (a.name > b.name) {
                return 1;
            }
            return 0;
        });
    }
    async generateMeasureColumns(namedMeasures) {
        const columns = [];
        const measuresPath = new Map();
        for (const { name, type } of namedMeasures) {
            if (!measuresPath.has(name)) {
                const { measure: measureDefinition } = await (0, kuzzle_plugin_commons_1.ask)("ask:device-manager:model:measure:get", { type });
                const flattenMeasuresPath = Object.keys((0, __1.flattenObject)(measureDefinition.valuesMappings)).map((path) => path.replace(".type", "").replace(".properties", ""));
                measuresPath.set(name, flattenMeasuresPath);
            }
            for (const path of measuresPath.get(name)) {
                const header = `${name}.${path}`.replace(`${name}.${type}`, name);
                columns.push({
                    header,
                    isMeasure: true,
                    path: `_source.measures.${name}.values.${path}`,
                });
            }
        }
        return columns;
    }
}
exports.DigitalTwinExporter = DigitalTwinExporter;
//# sourceMappingURL=DigitalTwinExporter.js.map