"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DigitalTwinService = void 0;
const kuzzle_1 = require("kuzzle");
const kuzzle_plugin_commons_1 = require("kuzzle-plugin-commons");
const plugin_1 = require("../../plugin");
const BaseService_1 = require("./BaseService");
class DigitalTwinService extends BaseService_1.BaseService {
    constructor(plugin, targetCollection) {
        super(plugin);
        this.targetCollection = targetCollection;
        this.digitalTwinType =
            this.targetCollection === plugin_1.InternalCollection.ASSETS ? "asset" : "device";
        this.registerAskEvents();
    }
    registerAskEvents() {
        (0, kuzzle_plugin_commons_1.onAsk)(`ask:device-manager:${this.digitalTwinType}:get-last-measures`, (payload) => {
            return this.getLastMeasures(payload.engineId, payload.digitalTwinId);
        });
    }
    /**
     * Gets the last measures of a digital twin
     */
    async getLastMeasures(engineId, digitalTwinId, measureCount = 100) {
        const measures = await this.mGetLastMeasures(engineId, [digitalTwinId], measureCount);
        if (!(digitalTwinId in measures)) {
            throw new kuzzle_1.NotFoundError("No measure could be found for this digital twin");
        }
        return measures[digitalTwinId];
    }
    /**
     * Gets the last measures of multiple digital twins
     */
    async mGetLastMeasures(engineId, digitalTwinIds, measureCount = 100) {
        if (digitalTwinIds.length === 0) {
            return {};
        }
        const aggregationParameters = this.getAggregationQueryParameters(digitalTwinIds);
        const result = await this.sdk.document.search(engineId, plugin_1.InternalCollection.MEASURES, {
            aggregations: {
                byDigitalTwin: {
                    aggregations: {
                        byMeasureName: {
                            aggregations: {
                                lastMeasure: {
                                    top_hits: {
                                        size: 1,
                                        sort: { measuredAt: "desc" },
                                    },
                                },
                            },
                            terms: {
                                field: aggregationParameters.measureNameField,
                                size: measureCount,
                            },
                        },
                    },
                    terms: {
                        field: aggregationParameters.idField,
                        size: digitalTwinIds.length,
                    },
                },
            },
            query: aggregationParameters.query,
        }, {
            size: 0,
        });
        const aggregations = result.aggregations;
        return aggregations.byDigitalTwin.buckets.reduce((digitalTwinAccumulator, digitalTwinBucket) => {
            const measureContents = digitalTwinBucket.byMeasureName.buckets.reduce((measureAccumulator, measureBucket) => {
                return [
                    ...measureAccumulator,
                    ...measureBucket.lastMeasure.hits.hits.map((hit) => hit._source),
                ];
            }, []);
            return {
                ...digitalTwinAccumulator,
                [digitalTwinBucket.key]: measureContents,
            };
        }, {});
    }
    async getLastMeasuredAt(engineId, digitalTwinId) {
        const aggregationParameters = this.getAggregationQueryParameters([
            digitalTwinId,
        ]);
        const result = await this.sdk.document.search(engineId, plugin_1.InternalCollection.MEASURES, {
            _source: "measuredAt",
            query: aggregationParameters.query,
            sort: {
                measuredAt: "desc",
            },
        }, {
            size: 1,
        });
        if (result.hits.length < 1) {
            throw new kuzzle_1.NotFoundError("No measure could be found for this digital twin");
        }
        return result.hits[0]._source.measuredAt;
    }
    async mGetLastMeasuredAt(engineId, digitalTwinIds) {
        if (digitalTwinIds.length === 0) {
            return {};
        }
        const aggregationParameters = this.getAggregationQueryParameters(digitalTwinIds);
        const result = await this.sdk.document.search(engineId, plugin_1.InternalCollection.MEASURES, {
            aggregations: {
                byDigitalTwin: {
                    aggregations: {
                        lastMeasuredAt: {
                            max: {
                                field: "measuredAt",
                            },
                        },
                    },
                    terms: {
                        field: aggregationParameters.idField,
                        size: digitalTwinIds.length,
                    },
                },
            },
            query: aggregationParameters.query,
        }, {
            size: 0,
        });
        const aggregations = result.aggregations;
        return aggregations.byDigitalTwin.buckets.reduce((accumulator, bucket) => {
            return {
                ...accumulator,
                [bucket.key]: bucket.lastMeasuredAt.value,
            };
        }, {});
    }
    getAggregationQueryParameters(digitalTwinId) {
        switch (this.digitalTwinType) {
            case "asset":
                return {
                    idField: "asset._id",
                    measureNameField: "asset.measureName",
                    query: {
                        terms: {
                            "asset._id": digitalTwinId,
                        },
                    },
                };
            case "device":
                return {
                    idField: "origin._id",
                    measureNameField: "origin.measureName",
                    query: {
                        bool: {
                            must: [
                                { terms: { "origin._id": digitalTwinId } },
                                { term: { "origin.type": "device" } },
                            ],
                        },
                    },
                };
        }
    }
}
exports.DigitalTwinService = DigitalTwinService;
//# sourceMappingURL=DigitalTwinService.js.map